#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from ..object import Object


class WebAppInfo(Object):
    """Contains information about a `Web App <https://core.telegram.org/bots/webapps>`_.

    Parameters:
        url (``str``):
            An HTTPS URL of a Web App to be opened with additional data as specified in
            `Initializing Web Apps <https://core.telegram.org/bots/webapps#initializing-web-apps>`_.
    """

    def __init__(
        self, *,
        url: str,
    ):
        super().__init__()

        self.url = url



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class UpdateWebPage(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.Update`.

    Details:
        - Layer: ``177``
        - ID: ``7F891213``

    Parameters:
        webpage (:obj:`WebPage <pyrogram.raw.base.WebPage>`):
            N/A

        pts (``int`` ``32-bit``):
            N/A

        pts_count (``int`` ``32-bit``):
            N/A

    """

    __slots__: List[str] = ["webpage", "pts", "pts_count"]

    ID = 0x7f891213
    QUALNAME = "types.UpdateWebPage"

    def __init__(self, *, webpage: "raw.base.WebPage", pts: int, pts_count: int) -> None:
        self.webpage = webpage  # WebPage
        self.pts = pts  # int
        self.pts_count = pts_count  # int

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "UpdateWebPage":
        # No flags
        
        webpage = TLObject.read(b)
        
        pts = Int.read(b)
        
        pts_count = Int.read(b)
        
        return UpdateWebPage(webpage=webpage, pts=pts, pts_count=pts_count)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        # No flags
        
        b.write(self.webpage.write())
        
        b.write(Int(self.pts))
        
        b.write(Int(self.pts_count))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class ResolvedBusinessChatLinks(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.account.ResolvedBusinessChatLinks`.

    Details:
        - Layer: ``177``
        - ID: ``9A23AF21``

    Parameters:
        peer (:obj:`Peer <pyrogram.raw.base.Peer>`):
            N/A

        message (``str``):
            N/A

        chats (List of :obj:`Chat <pyrogram.raw.base.Chat>`):
            N/A

        users (List of :obj:`User <pyrogram.raw.base.User>`):
            N/A

        entities (List of :obj:`MessageEntity <pyrogram.raw.base.MessageEntity>`, *optional*):
            N/A

    Functions:
        This object can be returned by 1 function.

        .. currentmodule:: pyrogram.raw.functions

        .. autosummary::
            :nosignatures:

            account.ResolveBusinessChatLink
    """

    __slots__: List[str] = ["peer", "message", "chats", "users", "entities"]

    ID = 0x9a23af21
    QUALNAME = "types.account.ResolvedBusinessChatLinks"

    def __init__(self, *, peer: "raw.base.Peer", message: str, chats: List["raw.base.Chat"], users: List["raw.base.User"], entities: Optional[List["raw.base.MessageEntity"]] = None) -> None:
        self.peer = peer  # Peer
        self.message = message  # string
        self.chats = chats  # Vector<Chat>
        self.users = users  # Vector<User>
        self.entities = entities  # flags.0?Vector<MessageEntity>

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "ResolvedBusinessChatLinks":
        
        flags = Int.read(b)
        
        peer = TLObject.read(b)
        
        message = String.read(b)
        
        entities = TLObject.read(b) if flags & (1 << 0) else []
        
        chats = TLObject.read(b)
        
        users = TLObject.read(b)
        
        return ResolvedBusinessChatLinks(peer=peer, message=message, chats=chats, users=users, entities=entities)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.entities else 0
        b.write(Int(flags))
        
        b.write(self.peer.write())
        
        b.write(String(self.message))
        
        if self.entities is not None:
            b.write(Vector(self.entities))
        
        b.write(Vector(self.chats))
        
        b.write(Vector(self.users))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

import pyrogram
from pyrogram import raw, utils
from pyrogram import types
from ..object import Object


class GameHighScore(Object):
    """One row of the high scores table for a game.

    Parameters:
        user (:obj:`~pyrogram.types.User`):
            User.

        score (``int``):
            Score.

        position (``int``, *optional*):
            Position in high score table for the game.
    """

    def __init__(
        self,
        *,
        client: "pyrogram.Client" = None,
        user: "types.User",
        score: int,
        position: int = None
    ):
        super().__init__(client)

        self.user = user
        self.score = score
        self.position = position

    @staticmethod
    def _parse(client, game_high_score: raw.types.HighScore, users: dict) -> "GameHighScore":
        users = {i.id: i for i in users}

        return GameHighScore(
            user=types.User._parse(client, users[game_high_score.user_id]),
            score=game_high_score.score,
            position=game_high_score.pos,
            client=client
        )

    @staticmethod
    def _parse_action(client, service: raw.types.MessageService, users: dict):
        return GameHighScore(
            user=types.User._parse(client, users[utils.get_raw_peer_id(service.from_id or service.peer_id)]),
            score=service.action.score,
            client=client
        )



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #

from .send_custom_request import SendCustomRequest
from .answer_webhook_json_query import AnswerWebhookJSONQuery
from .set_bot_commands import SetBotCommands
from .reset_bot_commands import ResetBotCommands
from .get_bot_commands import GetBotCommands
from .set_bot_menu_button import SetBotMenuButton
from .get_bot_menu_button import GetBotMenuButton
from .set_bot_broadcast_default_admin_rights import SetBotBroadcastDefaultAdminRights
from .set_bot_group_default_admin_rights import SetBotGroupDefaultAdminRights
from .set_bot_info import SetBotInfo
from .get_bot_info import GetBotInfo
from .reorder_usernames import ReorderUsernames
from .toggle_username import ToggleUsername
from .can_send_message import CanSendMessage
from .allow_send_message import AllowSendMessage
from .invoke_web_view_custom_method import InvokeWebViewCustomMethod



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class StoryItemSkipped(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.StoryItem`.

    Details:
        - Layer: ``177``
        - ID: ``FFADC913``

    Parameters:
        id (``int`` ``32-bit``):
            N/A

        date (``int`` ``32-bit``):
            N/A

        expire_date (``int`` ``32-bit``):
            N/A

        close_friends (``bool``, *optional*):
            N/A

    """

    __slots__: List[str] = ["id", "date", "expire_date", "close_friends"]

    ID = 0xffadc913
    QUALNAME = "types.StoryItemSkipped"

    def __init__(self, *, id: int, date: int, expire_date: int, close_friends: Optional[bool] = None) -> None:
        self.id = id  # int
        self.date = date  # int
        self.expire_date = expire_date  # int
        self.close_friends = close_friends  # flags.8?true

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "StoryItemSkipped":
        
        flags = Int.read(b)
        
        close_friends = True if flags & (1 << 8) else False
        id = Int.read(b)
        
        date = Int.read(b)
        
        expire_date = Int.read(b)
        
        return StoryItemSkipped(id=id, date=date, expire_date=expire_date, close_friends=close_friends)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 8) if self.close_friends else 0
        b.write(Int(flags))
        
        b.write(Int(self.id))
        
        b.write(Int(self.date))
        
        b.write(Int(self.expire_date))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class Dialog(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.Dialog`.

    Details:
        - Layer: ``177``
        - ID: ``D58A08C6``

    Parameters:
        peer (:obj:`Peer <pyrogram.raw.base.Peer>`):
            N/A

        top_message (``int`` ``32-bit``):
            N/A

        read_inbox_max_id (``int`` ``32-bit``):
            N/A

        read_outbox_max_id (``int`` ``32-bit``):
            N/A

        unread_count (``int`` ``32-bit``):
            N/A

        unread_mentions_count (``int`` ``32-bit``):
            N/A

        unread_reactions_count (``int`` ``32-bit``):
            N/A

        notify_settings (:obj:`PeerNotifySettings <pyrogram.raw.base.PeerNotifySettings>`):
            N/A

        pinned (``bool``, *optional*):
            N/A

        unread_mark (``bool``, *optional*):
            N/A

        view_forum_as_messages (``bool``, *optional*):
            N/A

        pts (``int`` ``32-bit``, *optional*):
            N/A

        draft (:obj:`DraftMessage <pyrogram.raw.base.DraftMessage>`, *optional*):
            N/A

        folder_id (``int`` ``32-bit``, *optional*):
            N/A

        ttl_period (``int`` ``32-bit``, *optional*):
            N/A

    """

    __slots__: List[str] = ["peer", "top_message", "read_inbox_max_id", "read_outbox_max_id", "unread_count", "unread_mentions_count", "unread_reactions_count", "notify_settings", "pinned", "unread_mark", "view_forum_as_messages", "pts", "draft", "folder_id", "ttl_period"]

    ID = 0xd58a08c6
    QUALNAME = "types.Dialog"

    def __init__(self, *, peer: "raw.base.Peer", top_message: int, read_inbox_max_id: int, read_outbox_max_id: int, unread_count: int, unread_mentions_count: int, unread_reactions_count: int, notify_settings: "raw.base.PeerNotifySettings", pinned: Optional[bool] = None, unread_mark: Optional[bool] = None, view_forum_as_messages: Optional[bool] = None, pts: Optional[int] = None, draft: "raw.base.DraftMessage" = None, folder_id: Optional[int] = None, ttl_period: Optional[int] = None) -> None:
        self.peer = peer  # Peer
        self.top_message = top_message  # int
        self.read_inbox_max_id = read_inbox_max_id  # int
        self.read_outbox_max_id = read_outbox_max_id  # int
        self.unread_count = unread_count  # int
        self.unread_mentions_count = unread_mentions_count  # int
        self.unread_reactions_count = unread_reactions_count  # int
        self.notify_settings = notify_settings  # PeerNotifySettings
        self.pinned = pinned  # flags.2?true
        self.unread_mark = unread_mark  # flags.3?true
        self.view_forum_as_messages = view_forum_as_messages  # flags.6?true
        self.pts = pts  # flags.0?int
        self.draft = draft  # flags.1?DraftMessage
        self.folder_id = folder_id  # flags.4?int
        self.ttl_period = ttl_period  # flags.5?int

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "Dialog":
        
        flags = Int.read(b)
        
        pinned = True if flags & (1 << 2) else False
        unread_mark = True if flags & (1 << 3) else False
        view_forum_as_messages = True if flags & (1 << 6) else False
        peer = TLObject.read(b)
        
        top_message = Int.read(b)
        
        read_inbox_max_id = Int.read(b)
        
        read_outbox_max_id = Int.read(b)
        
        unread_count = Int.read(b)
        
        unread_mentions_count = Int.read(b)
        
        unread_reactions_count = Int.read(b)
        
        notify_settings = TLObject.read(b)
        
        pts = Int.read(b) if flags & (1 << 0) else None
        draft = TLObject.read(b) if flags & (1 << 1) else None
        
        folder_id = Int.read(b) if flags & (1 << 4) else None
        ttl_period = Int.read(b) if flags & (1 << 5) else None
        return Dialog(peer=peer, top_message=top_message, read_inbox_max_id=read_inbox_max_id, read_outbox_max_id=read_outbox_max_id, unread_count=unread_count, unread_mentions_count=unread_mentions_count, unread_reactions_count=unread_reactions_count, notify_settings=notify_settings, pinned=pinned, unread_mark=unread_mark, view_forum_as_messages=view_forum_as_messages, pts=pts, draft=draft, folder_id=folder_id, ttl_period=ttl_period)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 2) if self.pinned else 0
        flags |= (1 << 3) if self.unread_mark else 0
        flags |= (1 << 6) if self.view_forum_as_messages else 0
        flags |= (1 << 0) if self.pts is not None else 0
        flags |= (1 << 1) if self.draft is not None else 0
        flags |= (1 << 4) if self.folder_id is not None else 0
        flags |= (1 << 5) if self.ttl_period is not None else 0
        b.write(Int(flags))
        
        b.write(self.peer.write())
        
        b.write(Int(self.top_message))
        
        b.write(Int(self.read_inbox_max_id))
        
        b.write(Int(self.read_outbox_max_id))
        
        b.write(Int(self.unread_count))
        
        b.write(Int(self.unread_mentions_count))
        
        b.write(Int(self.unread_reactions_count))
        
        b.write(self.notify_settings.write())
        
        if self.pts is not None:
            b.write(Int(self.pts))
        
        if self.draft is not None:
            b.write(self.draft.write())
        
        if self.folder_id is not None:
            b.write(Int(self.folder_id))
        
        if self.ttl_period is not None:
            b.write(Int(self.ttl_period))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class InputMessagesFilterPhoneCalls(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.MessagesFilter`.

    Details:
        - Layer: ``177``
        - ID: ``80C99768``

    Parameters:
        missed (``bool``, *optional*):
            N/A

    """

    __slots__: List[str] = ["missed"]

    ID = 0x80c99768
    QUALNAME = "types.InputMessagesFilterPhoneCalls"

    def __init__(self, *, missed: Optional[bool] = None) -> None:
        self.missed = missed  # flags.0?true

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "InputMessagesFilterPhoneCalls":
        
        flags = Int.read(b)
        
        missed = True if flags & (1 << 0) else False
        return InputMessagesFilterPhoneCalls(missed=missed)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.missed else 0
        b.write(Int(flags))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class InputPeerNotifySettings(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.InputPeerNotifySettings`.

    Details:
        - Layer: ``177``
        - ID: ``CACB6AE2``

    Parameters:
        show_previews (``bool``, *optional*):
            N/A

        silent (``bool``, *optional*):
            N/A

        mute_until (``int`` ``32-bit``, *optional*):
            N/A

        sound (:obj:`NotificationSound <pyrogram.raw.base.NotificationSound>`, *optional*):
            N/A

        stories_muted (``bool``, *optional*):
            N/A

        stories_hide_sender (``bool``, *optional*):
            N/A

        stories_sound (:obj:`NotificationSound <pyrogram.raw.base.NotificationSound>`, *optional*):
            N/A

    """

    __slots__: List[str] = ["show_previews", "silent", "mute_until", "sound", "stories_muted", "stories_hide_sender", "stories_sound"]

    ID = 0xcacb6ae2
    QUALNAME = "types.InputPeerNotifySettings"

    def __init__(self, *, show_previews: Optional[bool] = None, silent: Optional[bool] = None, mute_until: Optional[int] = None, sound: "raw.base.NotificationSound" = None, stories_muted: Optional[bool] = None, stories_hide_sender: Optional[bool] = None, stories_sound: "raw.base.NotificationSound" = None) -> None:
        self.show_previews = show_previews  # flags.0?Bool
        self.silent = silent  # flags.1?Bool
        self.mute_until = mute_until  # flags.2?int
        self.sound = sound  # flags.3?NotificationSound
        self.stories_muted = stories_muted  # flags.6?Bool
        self.stories_hide_sender = stories_hide_sender  # flags.7?Bool
        self.stories_sound = stories_sound  # flags.8?NotificationSound

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "InputPeerNotifySettings":
        
        flags = Int.read(b)
        
        show_previews = Bool.read(b) if flags & (1 << 0) else None
        silent = Bool.read(b) if flags & (1 << 1) else None
        mute_until = Int.read(b) if flags & (1 << 2) else None
        sound = TLObject.read(b) if flags & (1 << 3) else None
        
        stories_muted = Bool.read(b) if flags & (1 << 6) else None
        stories_hide_sender = Bool.read(b) if flags & (1 << 7) else None
        stories_sound = TLObject.read(b) if flags & (1 << 8) else None
        
        return InputPeerNotifySettings(show_previews=show_previews, silent=silent, mute_until=mute_until, sound=sound, stories_muted=stories_muted, stories_hide_sender=stories_hide_sender, stories_sound=stories_sound)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.show_previews is not None else 0
        flags |= (1 << 1) if self.silent is not None else 0
        flags |= (1 << 2) if self.mute_until is not None else 0
        flags |= (1 << 3) if self.sound is not None else 0
        flags |= (1 << 6) if self.stories_muted is not None else 0
        flags |= (1 << 7) if self.stories_hide_sender is not None else 0
        flags |= (1 << 8) if self.stories_sound is not None else 0
        b.write(Int(flags))
        
        if self.show_previews is not None:
            b.write(Bool(self.show_previews))
        
        if self.silent is not None:
            b.write(Bool(self.silent))
        
        if self.mute_until is not None:
            b.write(Int(self.mute_until))
        
        if self.sound is not None:
            b.write(self.sound.write())
        
        if self.stories_muted is not None:
            b.write(Bool(self.stories_muted))
        
        if self.stories_hide_sender is not None:
            b.write(Bool(self.stories_hide_sender))
        
        if self.stories_sound is not None:
            b.write(self.stories_sound.write())
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from pyrogram import raw
from pyrogram.session.internals import MsgId
from ..object import Object


class InputPhoneContact(Object):
    """A Phone Contact to be added in your Telegram address book.
    It is intended to be used with :meth:`~pyrogram.Client.add_contacts()`

    Parameters:
        phone (``str``):
            Contact's phone number

        first_name (``str``):
            Contact's first name

        last_name (``str``, *optional*):
            Contact's last name
    """

    def __init__(self, phone: str, first_name: str, last_name: str = ""):
        super().__init__(None)

    def __new__(cls,
                phone: str,
                first_name: str,
                last_name: str = ""):
        return raw.types.InputPhoneContact(
            client_id=MsgId(),
            phone="+" + phone.strip("+"),
            first_name=first_name,
            last_name=last_name
        )



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class DocumentAttributeVideo(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.DocumentAttribute`.

    Details:
        - Layer: ``177``
        - ID: ``D38FF1C2``

    Parameters:
        duration (``float`` ``64-bit``):
            N/A

        w (``int`` ``32-bit``):
            N/A

        h (``int`` ``32-bit``):
            N/A

        round_message (``bool``, *optional*):
            N/A

        supports_streaming (``bool``, *optional*):
            N/A

        nosound (``bool``, *optional*):
            N/A

        preload_prefix_size (``int`` ``32-bit``, *optional*):
            N/A

    """

    __slots__: List[str] = ["duration", "w", "h", "round_message", "supports_streaming", "nosound", "preload_prefix_size"]

    ID = 0xd38ff1c2
    QUALNAME = "types.DocumentAttributeVideo"

    def __init__(self, *, duration: float, w: int, h: int, round_message: Optional[bool] = None, supports_streaming: Optional[bool] = None, nosound: Optional[bool] = None, preload_prefix_size: Optional[int] = None) -> None:
        self.duration = duration  # double
        self.w = w  # int
        self.h = h  # int
        self.round_message = round_message  # flags.0?true
        self.supports_streaming = supports_streaming  # flags.1?true
        self.nosound = nosound  # flags.3?true
        self.preload_prefix_size = preload_prefix_size  # flags.2?int

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "DocumentAttributeVideo":
        
        flags = Int.read(b)
        
        round_message = True if flags & (1 << 0) else False
        supports_streaming = True if flags & (1 << 1) else False
        nosound = True if flags & (1 << 3) else False
        duration = Double.read(b)
        
        w = Int.read(b)
        
        h = Int.read(b)
        
        preload_prefix_size = Int.read(b) if flags & (1 << 2) else None
        return DocumentAttributeVideo(duration=duration, w=w, h=h, round_message=round_message, supports_streaming=supports_streaming, nosound=nosound, preload_prefix_size=preload_prefix_size)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.round_message else 0
        flags |= (1 << 1) if self.supports_streaming else 0
        flags |= (1 << 3) if self.nosound else 0
        flags |= (1 << 2) if self.preload_prefix_size is not None else 0
        b.write(Int(flags))
        
        b.write(Double(self.duration))
        
        b.write(Int(self.w))
        
        b.write(Int(self.h))
        
        if self.preload_prefix_size is not None:
            b.write(Int(self.preload_prefix_size))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class GlobalPrivacySettings(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.GlobalPrivacySettings`.

    Details:
        - Layer: ``177``
        - ID: ``734C4CCB``

    Parameters:
        archive_and_mute_new_noncontact_peers (``bool``, *optional*):
            N/A

        keep_archived_unmuted (``bool``, *optional*):
            N/A

        keep_archived_folders (``bool``, *optional*):
            N/A

        hide_read_marks (``bool``, *optional*):
            N/A

        new_noncontact_peers_require_premium (``bool``, *optional*):
            N/A

    Functions:
        This object can be returned by 2 functions.

        .. currentmodule:: pyrogram.raw.functions

        .. autosummary::
            :nosignatures:

            account.GetGlobalPrivacySettings
            account.SetGlobalPrivacySettings
    """

    __slots__: List[str] = ["archive_and_mute_new_noncontact_peers", "keep_archived_unmuted", "keep_archived_folders", "hide_read_marks", "new_noncontact_peers_require_premium"]

    ID = 0x734c4ccb
    QUALNAME = "types.GlobalPrivacySettings"

    def __init__(self, *, archive_and_mute_new_noncontact_peers: Optional[bool] = None, keep_archived_unmuted: Optional[bool] = None, keep_archived_folders: Optional[bool] = None, hide_read_marks: Optional[bool] = None, new_noncontact_peers_require_premium: Optional[bool] = None) -> None:
        self.archive_and_mute_new_noncontact_peers = archive_and_mute_new_noncontact_peers  # flags.0?true
        self.keep_archived_unmuted = keep_archived_unmuted  # flags.1?true
        self.keep_archived_folders = keep_archived_folders  # flags.2?true
        self.hide_read_marks = hide_read_marks  # flags.3?true
        self.new_noncontact_peers_require_premium = new_noncontact_peers_require_premium  # flags.4?true

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "GlobalPrivacySettings":
        
        flags = Int.read(b)
        
        archive_and_mute_new_noncontact_peers = True if flags & (1 << 0) else False
        keep_archived_unmuted = True if flags & (1 << 1) else False
        keep_archived_folders = True if flags & (1 << 2) else False
        hide_read_marks = True if flags & (1 << 3) else False
        new_noncontact_peers_require_premium = True if flags & (1 << 4) else False
        return GlobalPrivacySettings(archive_and_mute_new_noncontact_peers=archive_and_mute_new_noncontact_peers, keep_archived_unmuted=keep_archived_unmuted, keep_archived_folders=keep_archived_folders, hide_read_marks=hide_read_marks, new_noncontact_peers_require_premium=new_noncontact_peers_require_premium)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.archive_and_mute_new_noncontact_peers else 0
        flags |= (1 << 1) if self.keep_archived_unmuted else 0
        flags |= (1 << 2) if self.keep_archived_folders else 0
        flags |= (1 << 3) if self.hide_read_marks else 0
        flags |= (1 << 4) if self.new_noncontact_peers_require_premium else 0
        b.write(Int(flags))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

import logging
from binascii import crc32
from struct import pack, unpack
from typing import Optional

from .tcp import TCP

log = logging.getLogger(__name__)


class TCPFull(TCP):
    def __init__(self, ipv6: bool, proxy: dict):
        super().__init__(ipv6, proxy)

        self.seq_no = None

    async def connect(self, address: tuple):
        await super().connect(address)
        self.seq_no = 0

    async def send(self, data: bytes, *args):
        data = pack("<II", len(data) + 12, self.seq_no) + data
        data += pack("<I", crc32(data))
        self.seq_no += 1

        await super().send(data)

    async def recv(self, length: int = 0) -> Optional[bytes]:
        length = await super().recv(4)

        if length is None:
            return None

        packet = await super().recv(unpack("<I", length)[0] - 4)

        if packet is None:
            return None

        packet = length + packet
        checksum = packet[-4:]
        packet = packet[:-4]

        if crc32(packet) != unpack("<I", checksum)[0]:
            return None

        return packet[8:]



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class StoryView(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.StoryView`.

    Details:
        - Layer: ``177``
        - ID: ``B0BDEAC5``

    Parameters:
        user_id (``int`` ``64-bit``):
            N/A

        date (``int`` ``32-bit``):
            N/A

        blocked (``bool``, *optional*):
            N/A

        blocked_my_stories_from (``bool``, *optional*):
            N/A

        reaction (:obj:`Reaction <pyrogram.raw.base.Reaction>`, *optional*):
            N/A

    """

    __slots__: List[str] = ["user_id", "date", "blocked", "blocked_my_stories_from", "reaction"]

    ID = 0xb0bdeac5
    QUALNAME = "types.StoryView"

    def __init__(self, *, user_id: int, date: int, blocked: Optional[bool] = None, blocked_my_stories_from: Optional[bool] = None, reaction: "raw.base.Reaction" = None) -> None:
        self.user_id = user_id  # long
        self.date = date  # int
        self.blocked = blocked  # flags.0?true
        self.blocked_my_stories_from = blocked_my_stories_from  # flags.1?true
        self.reaction = reaction  # flags.2?Reaction

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "StoryView":
        
        flags = Int.read(b)
        
        blocked = True if flags & (1 << 0) else False
        blocked_my_stories_from = True if flags & (1 << 1) else False
        user_id = Long.read(b)
        
        date = Int.read(b)
        
        reaction = TLObject.read(b) if flags & (1 << 2) else None
        
        return StoryView(user_id=user_id, date=date, blocked=blocked, blocked_my_stories_from=blocked_my_stories_from, reaction=reaction)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.blocked else 0
        flags |= (1 << 1) if self.blocked_my_stories_from else 0
        flags |= (1 << 2) if self.reaction is not None else 0
        b.write(Int(flags))
        
        b.write(Long(self.user_id))
        
        b.write(Int(self.date))
        
        if self.reaction is not None:
            b.write(self.reaction.write())
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class ChangeAuthorizationSettings(TLObject):  # type: ignore
    """Telegram API function.

    Details:
        - Layer: ``177``
        - ID: ``40F48462``

    Parameters:
        hash (``int`` ``64-bit``):
            N/A

        confirmed (``bool``, *optional*):
            N/A

        encrypted_requests_disabled (``bool``, *optional*):
            N/A

        call_requests_disabled (``bool``, *optional*):
            N/A

    Returns:
        ``bool``
    """

    __slots__: List[str] = ["hash", "confirmed", "encrypted_requests_disabled", "call_requests_disabled"]

    ID = 0x40f48462
    QUALNAME = "functions.account.ChangeAuthorizationSettings"

    def __init__(self, *, hash: int, confirmed: Optional[bool] = None, encrypted_requests_disabled: Optional[bool] = None, call_requests_disabled: Optional[bool] = None) -> None:
        self.hash = hash  # long
        self.confirmed = confirmed  # flags.3?true
        self.encrypted_requests_disabled = encrypted_requests_disabled  # flags.0?Bool
        self.call_requests_disabled = call_requests_disabled  # flags.1?Bool

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "ChangeAuthorizationSettings":
        
        flags = Int.read(b)
        
        confirmed = True if flags & (1 << 3) else False
        hash = Long.read(b)
        
        encrypted_requests_disabled = Bool.read(b) if flags & (1 << 0) else None
        call_requests_disabled = Bool.read(b) if flags & (1 << 1) else None
        return ChangeAuthorizationSettings(hash=hash, confirmed=confirmed, encrypted_requests_disabled=encrypted_requests_disabled, call_requests_disabled=call_requests_disabled)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 3) if self.confirmed else 0
        flags |= (1 << 0) if self.encrypted_requests_disabled is not None else 0
        flags |= (1 << 1) if self.call_requests_disabled is not None else 0
        b.write(Int(flags))
        
        b.write(Long(self.hash))
        
        if self.encrypted_requests_disabled is not None:
            b.write(Bool(self.encrypted_requests_disabled))
        
        if self.call_requests_disabled is not None:
            b.write(Bool(self.call_requests_disabled))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class UpdateBotChatInviteRequester(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.Update`.

    Details:
        - Layer: ``177``
        - ID: ``11DFA986``

    Parameters:
        peer (:obj:`Peer <pyrogram.raw.base.Peer>`):
            N/A

        date (``int`` ``32-bit``):
            N/A

        user_id (``int`` ``64-bit``):
            N/A

        about (``str``):
            N/A

        invite (:obj:`ExportedChatInvite <pyrogram.raw.base.ExportedChatInvite>`):
            N/A

        qts (``int`` ``32-bit``):
            N/A

    """

    __slots__: List[str] = ["peer", "date", "user_id", "about", "invite", "qts"]

    ID = 0x11dfa986
    QUALNAME = "types.UpdateBotChatInviteRequester"

    def __init__(self, *, peer: "raw.base.Peer", date: int, user_id: int, about: str, invite: "raw.base.ExportedChatInvite", qts: int) -> None:
        self.peer = peer  # Peer
        self.date = date  # int
        self.user_id = user_id  # long
        self.about = about  # string
        self.invite = invite  # ExportedChatInvite
        self.qts = qts  # int

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "UpdateBotChatInviteRequester":
        # No flags
        
        peer = TLObject.read(b)
        
        date = Int.read(b)
        
        user_id = Long.read(b)
        
        about = String.read(b)
        
        invite = TLObject.read(b)
        
        qts = Int.read(b)
        
        return UpdateBotChatInviteRequester(peer=peer, date=date, user_id=user_id, about=about, invite=invite, qts=qts)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        # No flags
        
        b.write(self.peer.write())
        
        b.write(Int(self.date))
        
        b.write(Long(self.user_id))
        
        b.write(String(self.about))
        
        b.write(self.invite.write())
        
        b.write(Int(self.qts))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class UpdateMessagePoll(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.Update`.

    Details:
        - Layer: ``177``
        - ID: ``ACA1657B``

    Parameters:
        poll_id (``int`` ``64-bit``):
            N/A

        results (:obj:`PollResults <pyrogram.raw.base.PollResults>`):
            N/A

        poll (:obj:`Poll <pyrogram.raw.base.Poll>`, *optional*):
            N/A

    """

    __slots__: List[str] = ["poll_id", "results", "poll"]

    ID = 0xaca1657b
    QUALNAME = "types.UpdateMessagePoll"

    def __init__(self, *, poll_id: int, results: "raw.base.PollResults", poll: "raw.base.Poll" = None) -> None:
        self.poll_id = poll_id  # long
        self.results = results  # PollResults
        self.poll = poll  # flags.0?Poll

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "UpdateMessagePoll":
        
        flags = Int.read(b)
        
        poll_id = Long.read(b)
        
        poll = TLObject.read(b) if flags & (1 << 0) else None
        
        results = TLObject.read(b)
        
        return UpdateMessagePoll(poll_id=poll_id, results=results, poll=poll)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.poll is not None else 0
        b.write(Int(flags))
        
        b.write(Long(self.poll_id))
        
        if self.poll is not None:
            b.write(self.poll.write())
        
        b.write(self.results.write())
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from .ask import Ask
from .get_listener_matching_with_data import GetListenerMatchingWithData
from .get_listener_matching_with_identifier_pattern import GetListenerMatchingWithIdentifierPattern
from .get_many_listeners_matching_with_data import GetManyListenersMatchingWithData
from .get_many_listeners_matching_with_identifier_pattern import GetManyListenersMatchingWithIdentifierPattern
from .listen import Listen
from .register_next_step_handler import RegisterNextStepHandler
from .remove_listerner import RemoveListener
from .stop_listener import StopListener
from .stop_listening import StopListening
from .wait_for_callback_query import WaitForCallbackQuery
from .wait_for_message import WaitForMessage

class Pyromod(
    Ask,
    GetListenerMatchingWithData,
    GetListenerMatchingWithIdentifierPattern,
    GetManyListenersMatchingWithData,
    GetManyListenersMatchingWithIdentifierPattern,
    Listen,
    RegisterNextStepHandler,
    RemoveListener,
    StopListener,
    StopListening,
    WaitForCallbackQuery,
    WaitForMessage
):
    pass



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class UpdateBotEditBusinessMessage(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.Update`.

    Details:
        - Layer: ``177``
        - ID: ``7DF587C``

    Parameters:
        connection_id (``str``):
            N/A

        message (:obj:`Message <pyrogram.raw.base.Message>`):
            N/A

        qts (``int`` ``32-bit``):
            N/A

        reply_to_message (:obj:`Message <pyrogram.raw.base.Message>`, *optional*):
            N/A

    """

    __slots__: List[str] = ["connection_id", "message", "qts", "reply_to_message"]

    ID = 0x7df587c
    QUALNAME = "types.UpdateBotEditBusinessMessage"

    def __init__(self, *, connection_id: str, message: "raw.base.Message", qts: int, reply_to_message: "raw.base.Message" = None) -> None:
        self.connection_id = connection_id  # string
        self.message = message  # Message
        self.qts = qts  # int
        self.reply_to_message = reply_to_message  # flags.0?Message

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "UpdateBotEditBusinessMessage":
        
        flags = Int.read(b)
        
        connection_id = String.read(b)
        
        message = TLObject.read(b)
        
        reply_to_message = TLObject.read(b) if flags & (1 << 0) else None
        
        qts = Int.read(b)
        
        return UpdateBotEditBusinessMessage(connection_id=connection_id, message=message, qts=qts, reply_to_message=reply_to_message)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.reply_to_message is not None else 0
        b.write(Int(flags))
        
        b.write(String(self.connection_id))
        
        b.write(self.message.write())
        
        if self.reply_to_message is not None:
            b.write(self.reply_to_message.write())
        
        b.write(Int(self.qts))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

import asyncio
import os
from datetime import datetime
from typing import Union, Optional, Callable, BinaryIO

import pyrogram
from pyrogram import types
from pyrogram.file_id import FileId, FileType, PHOTO_TYPES

DEFAULT_DOWNLOAD_DIR = "downloads/"


class DownloadMedia:
    async def download_media(
        self: "pyrogram.Client",
        message: Union["types.Message", "types.Story", str],
        file_name: str = DEFAULT_DOWNLOAD_DIR,
        in_memory: bool = False,
        block: bool = True,
        progress: Callable = None,
        progress_args: tuple = ()
    ) -> Optional[Union[str, BinaryIO]]:
        """Download the media from a message.

        .. include:: /_includes/usable-by/users-bots.rst

        Parameters:
            message (:obj:`~pyrogram.types.Message` | :obj:`~pyrogram.types.Story` | ``str``):
                Pass a Message/Story containing the media, the media itself (message.audio, message.video, ...) or a file id
                as string.

            file_name (``str``, *optional*):
                A custom *file_name* to be used instead of the one provided by Telegram.
                By default, all files are downloaded in the *downloads* folder in your working directory.
                You can also specify a path for downloading files in a custom location: paths that end with "/"
                are considered directories. All non-existent folders will be created automatically.

            in_memory (``bool``, *optional*):
                Pass True to download the media in-memory.
                A binary file-like object with its attribute ".name" set will be returned.
                Defaults to False.

            block (``bool``, *optional*):
                Blocks the code execution until the file has been downloaded.
                Defaults to True.

            progress (``Callable``, *optional*):
                Pass a callback function to view the file transmission progress.
                The function must take *(current, total)* as positional arguments (look at Other Parameters below for a
                detailed description) and will be called back each time a new file chunk has been successfully
                transmitted.

            progress_args (``tuple``, *optional*):
                Extra custom arguments for the progress callback function.
                You can pass anything you need to be available in the progress callback scope; for example, a Message
                object or a Client instance in order to edit the message with the updated progress status.

        Other Parameters:
            current (``int``):
                The amount of bytes transmitted so far.

            total (``int``):
                The total size of the file.

            *args (``tuple``, *optional*):
                Extra custom arguments as defined in the ``progress_args`` parameter.
                You can either keep ``*args`` or add every single extra argument in your function signature.

        Returns:
            ``str`` | ``None`` | ``BinaryIO``: On success, the absolute path of the downloaded file is returned,
            otherwise, in case the download failed or was deliberately stopped with
            :meth:`~pyrogram.Client.stop_transmission`, None is returned.
            Otherwise, in case ``in_memory=True``, a binary file-like object with its attribute ".name" set is returned.

        Raises:
            ValueError: if the message doesn't contain any downloadable media

        Example:
            Download media to file

            .. code-block:: python

                # Download from Message
                await app.download_media(message)

                # Download from file id
                await app.download_media(message.photo.file_id)

                # Keep track of the progress while downloading
                async def progress(current, total):
                    print(f"{current * 100 / total:.1f}%")

                await app.download_media(message, progress=progress)

            Download media in-memory

            .. code-block:: python

                file = await app.download_media(message, in_memory=True)

                file_name = file.name
                file_bytes = bytes(file.getbuffer())
        """
        available_media = ("audio", "document", "photo", "sticker", "animation", "video", "voice", "video_note",
                           "new_chat_photo")

        if isinstance(message, types.Message) or isinstance(message, types.Story):
            for kind in available_media:
                media = getattr(message, kind, None)

                if media is not None:
                    break
            else:
                raise ValueError("This message doesn't contain any downloadable media")
        else:
            media = message

        if isinstance(media, str):
            file_id_str = media
        else:
            file_id_str = media.file_id

        file_id_obj = FileId.decode(file_id_str)

        file_type = file_id_obj.file_type
        media_file_name = getattr(media, "file_name", "")
        file_size = getattr(media, "file_size", 0)
        mime_type = getattr(media, "mime_type", "")
        date = getattr(media, "date", None)

        directory, file_name = os.path.split(file_name)
        file_name = file_name or media_file_name or ""

        if not os.path.isabs(file_name):
            directory = self.PARENT_DIR / (directory or DEFAULT_DOWNLOAD_DIR)

        if not file_name:
            guessed_extension = self.guess_extension(mime_type)

            if file_type in PHOTO_TYPES:
                extension = ".jpg"
            elif file_type == FileType.VOICE:
                extension = guessed_extension or ".ogg"
            elif file_type in (FileType.VIDEO, FileType.ANIMATION, FileType.VIDEO_NOTE):
                extension = guessed_extension or ".mp4"
            elif file_type == FileType.DOCUMENT:
                extension = guessed_extension or ".zip"
            elif file_type == FileType.STICKER:
                extension = guessed_extension or ".webp"
            elif file_type == FileType.AUDIO:
                extension = guessed_extension or ".mp3"
            else:
                extension = ".unknown"

            file_name = "{}_{}_{}{}".format(
                FileType(file_id_obj.file_type).name.lower(),
                (date or datetime.now()).strftime("%Y-%m-%d_%H-%M-%S"),
                self.rnd_id(),
                extension
            )

        downloader = self.handle_download(
            (file_id_obj, directory, file_name, in_memory, file_size, progress, progress_args)
        )

        if block:
            return await downloader
        else:
            asyncio.get_event_loop().create_task(downloader)



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class SaveAutoSaveSettings(TLObject):  # type: ignore
    """Telegram API function.

    Details:
        - Layer: ``177``
        - ID: ``D69B8361``

    Parameters:
        settings (:obj:`AutoSaveSettings <pyrogram.raw.base.AutoSaveSettings>`):
            N/A

        users (``bool``, *optional*):
            N/A

        chats (``bool``, *optional*):
            N/A

        broadcasts (``bool``, *optional*):
            N/A

        peer (:obj:`InputPeer <pyrogram.raw.base.InputPeer>`, *optional*):
            N/A

    Returns:
        ``bool``
    """

    __slots__: List[str] = ["settings", "users", "chats", "broadcasts", "peer"]

    ID = 0xd69b8361
    QUALNAME = "functions.account.SaveAutoSaveSettings"

    def __init__(self, *, settings: "raw.base.AutoSaveSettings", users: Optional[bool] = None, chats: Optional[bool] = None, broadcasts: Optional[bool] = None, peer: "raw.base.InputPeer" = None) -> None:
        self.settings = settings  # AutoSaveSettings
        self.users = users  # flags.0?true
        self.chats = chats  # flags.1?true
        self.broadcasts = broadcasts  # flags.2?true
        self.peer = peer  # flags.3?InputPeer

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "SaveAutoSaveSettings":
        
        flags = Int.read(b)
        
        users = True if flags & (1 << 0) else False
        chats = True if flags & (1 << 1) else False
        broadcasts = True if flags & (1 << 2) else False
        peer = TLObject.read(b) if flags & (1 << 3) else None
        
        settings = TLObject.read(b)
        
        return SaveAutoSaveSettings(settings=settings, users=users, chats=chats, broadcasts=broadcasts, peer=peer)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.users else 0
        flags |= (1 << 1) if self.chats else 0
        flags |= (1 << 2) if self.broadcasts else 0
        flags |= (1 << 3) if self.peer is not None else 0
        b.write(Int(flags))
        
        if self.peer is not None:
            b.write(self.peer.write())
        
        b.write(self.settings.write())
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from typing import List, Union, BinaryIO

from ..messages_and_media import MessageEntity
from ..object import Object


class InputMedia(Object):
    """Content of a media message to be sent.

    It should be one of:

    - :obj:`~pyrogram.types.InputMediaAnimation`
    - :obj:`~pyrogram.types.InputMediaDocument`
    - :obj:`~pyrogram.types.InputMediaAudio`
    - :obj:`~pyrogram.types.InputMediaPhoto`
    - :obj:`~pyrogram.types.InputMediaVideo`
    """

    def __init__(
        self,
        media: Union[str, BinaryIO],
        caption: str = "",
        parse_mode: str = None,
        caption_entities: List[MessageEntity] = None
    ):
        super().__init__()

        self.media = media
        self.caption = caption
        self.parse_mode = parse_mode
        self.caption_entities = caption_entities



# Pyrofork - Telegram MTProto API Client Library for Python
# Copyright (C) 2017-present Dan <https://github.com/delivrance>
# Copyright (C) 2022-present Mayuri-Chan <https://github.com/Mayuri-Chan>
#
# This file is part of Pyrofork.
#
# Pyrofork is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyrofork is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
#ᴘʏʀᴏɢᴇʀᴀᴍ If not, see <http://www.gnu.org/licenses/>.

count = 596

exceptions = {
    503: {
        "_": "ServiceUnavailable",
        "ApiCallError": "ApiCallError",
        "Timeout": "Timeout",
        "Timedout": "Timedout",
    },
    403: {
        "_": "Forbidden",
        "BROADCAST_FORBIDDEN": "BroadcastForbidden",
        "CHANNEL_PUBLIC_GROUP_NA": "ChannelPublicGroupNa",
        "CHAT_ADMIN_INVITE_REQUIRED": "ChatAdminInviteRequired",
        "CHAT_ADMIN_REQUIRED": "ChatAdminRequired",
        "CHAT_FORBIDDEN": "ChatForbidden",
        "CHAT_GUEST_SEND_FORBIDDEN": "ChatGuestSendForbidden",
        "EDIT_BOT_INVITE_FORBIDDEN": "EditBotInviteForbidden",
        "INLINE_BOT_REQUIRED": "InlineBotRequired",
        "MESSAGE_AUTHOR_REQUIRED": "MessageAuthorRequired",
        "MESSAGE_DELETE_FORBIDDEN": "MessageDeleteForbidden",
        "NOT_ALLOWED": "NotAllowed",
        "PARTICIPANT_JOIN_MISSING": "ParticipantJoinMissing",
        "POLL_VOTE_REQUIRED": "PollVoteRequired",
        "PREMIUM_ACCOUNT_REQUIRED": "PremiumAccountRequired",
        "PRIVACY_PREMIUM_REQUIRED": "PrivacyPremiumRequired",
        "PUBLIC_CHANNEL_MISSING": "PublicChannelMissing",
        "RIGHT_FORBIDDEN": "RightForbidden",
        "SENSITIVE_CHANGE_FORBIDDEN": "SensitiveChangeForbidden",
        "TAKEOUT_REQUIRED": "TakeoutRequired",
        "USER_BOT_INVALID": "UserBotInvalid",
        "USER_CHANNELS_TOO_MUCH": "UserChannelsTooMuch",
        "USER_DELETED": "UserDeleted",
        "USER_INVALID": "UserInvalid",
        "USER_IS_BLOCKED": "UserIsBlocked",
        "USER_NOT_MUTUAL_CONTACT": "UserNotMutualContact",
        "USER_PRIVACY_RESTRICTED": "UserPrivacyRestricted",
        "USER_RESTRICTED": "UserRestricted",
        "CHAT_SEND_AUDIOS_FORBIDDEN": "ChatSendAudiosForbidden",
        "CHAT_SEND_DOCS_FORBIDDEN": "ChatSendDocsForbidden",
        "CHAT_SEND_GAME_FORBIDDEN": "ChatSendGameForbidden",
        "CHAT_SEND_GIFS_FORBIDDEN": "ChatSendGifsForbidden",
        "CHAT_SEND_INLINE_FORBIDDEN": "ChatSendInlineForbidden",
        "CHAT_SEND_MEDIA_FORBIDDEN": "ChatSendMediaForbidden",
        "CHAT_SEND_PHOTOS_FORBIDDEN": "ChatSendPhotosForbidden",
        "CHAT_SEND_PLAIN_FORBIDDEN": "ChatSendPlainForbidden",
        "CHAT_SEND_POLL_FORBIDDEN": "ChatSendPollForbidden",
        "CHAT_SEND_STICKERS_FORBIDDEN": "ChatSendStickersForbidden",
        "CHAT_SEND_VIDEOS_FORBIDDEN": "ChatSendVideosForbidden",
        "CHAT_SEND_VOICES_FORBIDDEN": "ChatSendVoicesForbidden",
        "CHAT_WRITE_FORBIDDEN": "ChatWriteForbidden",
        "GROUPCALL_ALREADY_STARTED": "GroupcallAlreadyStarted",
        "GROUPCALL_FORBIDDEN": "GroupcallForbidden",
        "LIVE_DISABLED": "LiveDisabled",
        "CHAT_GUEST_SEND_FORBIDDEN": "ChatGuestSendForbidden",
    },
    303: {
        "_": "SeeOther",
        "FILE_MIGRATE_X": "FileMigrate",
        "NETWORK_MIGRATE_X": "NetworkMigrate",
        "PHONE_MIGRATE_X": "PhoneMigrate",
        "STATS_MIGRATE_X": "StatsMigrate",
        "USER_MIGRATE_X": "UserMigrate",
    },
    420: {
        "_": "Flood",
        "2FA_CONFIRM_WAIT_X": "TwoFaConfirmWait",
        "FLOOD_TEST_PHONE_WAIT_X": "FloodTestPhoneWait",
        "FLOOD_WAIT_X": "FloodWait",
        "PREMIUM_SUB_ACTIVE_UNTIL_X": "PremiumSubActiveUntil",
        "SLOWMODE_WAIT_X": "SlowmodeWait",
        "STORY_SEND_FLOOD_X": "StorySendFlood",
        "TAKEOUT_INIT_DELAY_X": "TakeoutInitDelay",
    },
    401: {
        "_": "Unauthorized",
        "ACTIVE_USER_REQUIRED": "ActiveUserRequired",
        "AUTH_KEY_INVALID": "AuthKeyInvalid",
        "AUTH_KEY_PERM_EMPTY": "AuthKeyPermEmpty",
        "AUTH_KEY_UNREGISTERED": "AuthKeyUnregistered",
        "SESSION_EXPIRED": "SessionExpired",
        "SESSION_PASSWORD_NEEDED": "SessionPasswordNeeded",
        "SESSION_REVOKED": "SessionRevoked",
        "USER_DEACTIVATED": "UserDeactivated",
        "USER_DEACTIVATED_BAN": "UserDeactivatedBan",
    },
    406: {
        "_": "NotAcceptable",
        "AUTH_KEY_DUPLICATED": "AuthKeyDuplicated",
        "CHANNEL_PRIVATE": "ChannelPrivate",
        "CHANNEL_TOO_LARGE": "ChannelTooLarge",
        "CHAT_FORWARDS_RESTRICTED": "ChatForwardsRestricted",
        "FILEREF_UPGRADE_NEEDED": "FilerefUpgradeNeeded",
        "FRESH_CHANGE_ADMINS_FORBIDDEN": "FreshChangeAdminsForbidden",
        "FRESH_CHANGE_PHONE_FORBIDDEN": "FreshChangePhoneForbidden",
        "FRESH_RESET_AUTHORISATION_FORBIDDEN": "FreshResetAuthorisationForbidden",
        "GIFTCODE_NOT_ALLOWED": "GiftcodeNotAllowed",
        "INVITE_HASH_EXPIRED": "InviteHashExpired",
        "PHONE_NUMBER_INVALID": "PhoneNumberInvalid",
        "PHONE_PASSWORD_FLOOD": "PhonePasswordFlood",
        "PREMIUM_CURRENTLY_UNAVAILABLE": "PremiumCurrentlyUnavailable",
        "PREVIOUS_CHAT_IMPORT_ACTIVE_WAIT_XMIN": "PreviousChatImportActiveWaitMin",
        "SEND_CODE_UNAVAILABLE": "SendCodeUnavailable",
        "STICKERSET_INVALID": "StickersetInvalid",
        "STICKERSET_OWNER_ANONYMOUS": "StickersetOwnerAnonymous",
        "UPDATE_APP_TO_LOGIN": "UpdateAppToLogin",
        "USERPIC_PRIVACY_REQUIRED": "UserpicPrivacyRequired",
        "USERPIC_UPLOAD_REQUIRED": "UserpicUploadRequired",
        "USER_RESTRICTED": "UserRestricted",
    },
    400: {
        "_": "BadRequest",
        "ABOUT_TOO_LONG": "AboutTooLong",
        "ACCESS_TOKEN_EXPIRED": "AccessTokenExpired",
        "ACCESS_TOKEN_INVALID": "AccessTokenInvalid",
        "ADMINS_TOO_MUCH": "AdminsTooMuch",
        "ADMIN_ID_INVALID": "AdminIdInvalid",
        "ADMIN_RANK_EMOJI_NOT_ALLOWED": "AdminRankEmojiNotAllowed",
        "ADMIN_RANK_INVALID": "AdminRankInvalid",
        "ALBUM_PHOTOS_TOO_MANY": "AlbumPhotosTooMany",
        "API_ID_INVALID": "ApiIdInvalid",
        "API_ID_PUBLISHED_FLOOD": "ApiIdPublishedFlood",
        "ARTICLE_TITLE_EMPTY": "ArticleTitleEmpty",
        "AUDIO_CONTENT_URL_EMPTY": "AudioContentUrlEmpty",
        "AUDIO_TITLE_EMPTY": "AudioTitleEmpty",
        "AUTH_BYTES_INVALID": "AuthBytesInvalid",
        "AUTH_TOKEN_ALREADY_ACCEPTED": "AuthTokenAlreadyAccepted",
        "AUTH_TOKEN_EXCEPTION": "AuthTokenException",
        "AUTH_TOKEN_EXPIRED": "AuthTokenExpired",
        "AUTH_TOKEN_INVALID": "AuthTokenInvalid",
        "AUTH_TOKEN_INVALID2": "AuthTokenInvalid2",
        "AUTH_TOKEN_INVALIDX": "AuthTokenInvalidx",
        "AUTOARCHIVE_NOT_AVAILABLE": "AutoarchiveNotAvailable",
        "BANK_CARD_NUMBER_INVALID": "BankCardNumberInvalid",
        "BANNED_RIGHTS_INVALID": "BannedRightsInvalid",
        "BASE_PORT_LOC_INVALID": "BasePortLocInvalid",
        "BOTS_TOO_MUCH": "BotsTooMuch",
        "BOT_CHANNELS_NA": "BotChannelsNa",
        "BOT_COMMAND_DESCRIPTION_INVALID": "BotCommandDescriptionInvalid",
        "BOT_COMMAND_INVALID": "BotCommandInvalid",
        "BOT_DOMAIN_INVALID": "BotDomainInvalid",
        "BOT_GAMES_DISABLED": "BotGamesDisabled",
        "BOT_GROUPS_BLOCKED": "BotGroupsBlocked",
        "BOT_INLINE_DISABLED": "BotInlineDisabled",
        "BOT_INVALID": "BotInvalid",
        "BOT_METHOD_INVALID": "BotMethodInvalid",
        "BOT_MISSING": "BotMissing",
        "BOT_ONESIDE_NOT_AVAIL": "BotOnesideNotAvail",
        "BOT_PAYMENTS_DISABLED": "BotPaymentsDisabled",
        "BOT_POLLS_DISABLED": "BotPollsDisabled",
        "BOT_RESPONSE_TIMEOUT": "BotResponseTimeout",
        "BOT_SCORE_NOT_MODIFIED": "BotScoreNotModified",
        "BROADCAST_CALLS_DISABLED": "BroadcastCallsDisabled",
        "BROADCAST_ID_INVALID": "BroadcastIdInvalid",
        "BROADCAST_PUBLIC_VOTERS_FORBIDDEN": "BroadcastPublicVotersForbidden",
        "BROADCAST_REQUIRED": "BroadcastRequired",
        "BUTTON_DATA_INVALID": "ButtonDataInvalid",
        "BUTTON_TEXT_INVALID": "ButtonTextInvalid",
        "BUTTON_TYPE_INVALID": "ButtonTypeInvalid",
        "BUTTON_URL_INVALID": "ButtonUrlInvalid",
        "BUTTON_USER_PRIVACY_RESTRICTED": "ButtonUserPrivacyRestricted",
        "CALL_ALREADY_ACCEPTED": "CallAlreadyAccepted",
        "CALL_ALREADY_DECLINED": "CallAlreadyDeclined",
        "CALL_PEER_INVALID": "CallPeerInvalid",
        "CALL_PROTOCOL_FLAGS_INVALID": "CallProtocolFlagsInvalid",
        "CDN_METHOD_INVALID": "CdnMethodInvalid",
        "CHANNELS_ADMIN_LOCATED_TOO_MUCH": "ChannelsAdminLocatedTooMuch",
        "CHANNELS_ADMIN_PUBLIC_TOO_MUCH": "ChannelsAdminPublicTooMuch",
        "CHANNELS_TOO_MUCH": "ChannelsTooMuch",
        "CHANNEL_ADD_INVALID": "ChannelAddInvalid",
        "CHANNEL_BANNED": "ChannelBanned",
        "CHANNEL_INVALID": "ChannelInvalid",
        "CHANNEL_PARICIPANT_MISSING": "ChannelParicipantMissing",
        "CHANNEL_PRIVATE": "ChannelPrivate",
        "CHANNEL_TOO_BIG": "ChannelTooBig",
        "CHANNEL_TOO_LARGE": "ChannelTooLarge",
        "CHAT_ABOUT_NOT_MODIFIED": "ChatAboutNotModified",
        "CHAT_ABOUT_TOO_LONG": "ChatAboutTooLong",
        "CHAT_ADMIN_REQUIRED": "ChatAdminRequired",
        "CHAT_DISCUSSION_UNALLOWED": "ChatDiscussionUnallowed",
        "CHAT_FORWARDS_RESTRICTED": "ChatForwardsRestricted",
        "CHAT_ID_EMPTY": "ChatIdEmpty",
        "CHAT_ID_INVALID": "ChatIdInvalid",
        "CHAT_INVALID": "ChatInvalid",
        "CHAT_INVITE_PERMANENT": "ChatInvitePermanent",
        "CHAT_LINK_EXISTS": "ChatLinkExists",
        "CHAT_NOT_MODIFIED": "ChatNotModified",
        "CHAT_RESTRICTED": "ChatRestricted",
        "CHAT_REVOKE_DATE_UNSUPPORTED": "ChatRevokeDateUnsupported",
        "CHAT_SEND_INLINE_FORBIDDEN": "ChatSendInlineForbidden",
        "CHAT_TITLE_EMPTY": "ChatTitleEmpty",
        "CHAT_TOO_BIG": "ChatTooBig",
        "CODE_EMPTY": "CodeEmpty",
        "CODE_HASH_INVALID": "CodeHashInvalid",
        "CODE_INVALID": "CodeInvalid",
        "COLOR_INVALID": "ColorInvalid",
        "CONNECTION_API_ID_INVALID": "ConnectionApiIdInvalid",
        "CONNECTION_APP_VERSION_EMPTY": "ConnectionAppVersionEmpty",
        "CONNECTION_DEVICE_MODEL_EMPTY": "ConnectionDeviceModelEmpty",
        "CONNECTION_LANG_PACK_INVALID": "ConnectionLangPackInvalid",
        "CONNECTION_LAYER_INVALID": "ConnectionLayerInvalid",
        "CONNECTION_NOT_INITED": "ConnectionNotInited",
        "CONNECTION_SYSTEM_EMPTY": "ConnectionSystemEmpty",
        "CONNECTION_SYSTEM_LANG_CODE_EMPTY": "ConnectionSystemLangCodeEmpty",
        "CONTACT_ADD_MISSING": "ContactAddMissing",
        "CONTACT_ID_INVALID": "ContactIdInvalid",
        "CONTACT_NAME_EMPTY": "ContactNameEmpty",
        "CONTACT_REQ_MISSING": "ContactReqMissing",
        "CREATE_CALL_FAILED": "CreateCallFailed",
        "CURRENCY_TOTAL_AMOUNT_INVALID": "CurrencyTotalAmountInvalid",
        "DATA_INVALID": "DataInvalid",
        "DATA_JSON_INVALID": "DataJsonInvalid",
        "DATA_TOO_LONG": "DataTooLong",
        "DATE_EMPTY": "DateEmpty",
        "DC_ID_INVALID": "DcIdInvalid",
        "DH_G_A_INVALID": "DhGAInvalid",
        "DOCUMENT_INVALID": "DocumentInvalid",
        "EMAIL_HASH_EXPIRED": "EmailHashExpired",
        "EMAIL_INVALID": "EmailInvalid",
        "EMAIL_UNCONFIRMED": "EmailUnconfirmed",
        "EMAIL_UNCONFIRMED_X": "EmailUnconfirmed",
        "EMAIL_VERIFY_EXPIRED": "EmailVerifyExpired",
        "EMOJI_INVALID": "EmojiInvalid",
        "EMOJI_NOT_MODIFIED": "EmojiNotModified",
        "EMOTICON_EMPTY": "EmoticonEmpty",
        "EMOTICON_INVALID": "EmoticonInvalid",
        "EMOTICON_STICKERPACK_MISSING": "EmoticonStickerpackMissing",
        "ENCRYPTED_MESSAGE_INVALID": "EncryptedMessageInvalid",
        "ENCRYPTION_ALREADY_ACCEPTED": "EncryptionAlreadyAccepted",
        "ENCRYPTION_ALREADY_DECLINED": "EncryptionAlreadyDeclined",
        "ENCRYPTION_DECLINED": "EncryptionDeclined",
        "ENCRYPTION_ID_INVALID": "EncryptionIdInvalid",
        "ENTITIES_TOO_LONG": "EntitiesTooLong",
        "ENTITY_BOUNDS_INVALID": "EntityBoundsInvalid",
        "ENTITY_MENTION_USER_INVALID": "EntityMentionUserInvalid",
        "ERROR_TEXT_EMPTY": "ErrorTextEmpty",
        "EXPIRE_DATE_INVALID": "ExpireDateInvalid",
        "EXPIRE_FORBIDDEN": "ExpireForbidden",
        "EXPORT_CARD_INVALID": "ExportCardInvalid",
        "EXTERNAL_URL_INVALID": "ExternalUrlInvalid",
        "FIELD_NAME_EMPTY": "FieldNameEmpty",
        "FIELD_NAME_INVALID": "FieldNameInvalid",
        "FILE_CONTENT_TYPE_INVALID": "FileContentTypeInvalid",
        "FILE_EMTPY": "FileEmtpy",
        "FILE_ID_INVALID": "FileIdInvalid",
        "FILE_MIGRATE_X": "FileMigrate",
        "FILE_PARTS_INVALID": "FilePartsInvalid",
        "FILE_PART_0_MISSING": "FilePart0Missing",
        "FILE_PART_EMPTY": "FilePartEmpty",
        "FILE_PART_INVALID": "FilePartInvalid",
        "FILE_PART_LENGTH_INVALID": "FilePartLengthInvalid",
        "FILE_PART_SIZE_CHANGED": "FilePartSizeChanged",
        "FILE_PART_SIZE_INVALID": "FilePartSizeInvalid",
        "FILE_PART_TOO_BIG": "FilePartTooBig",
        "FILE_PART_X_MISSING": "FilePartMissing",
        "FILE_REFERENCE_EMPTY": "FileReferenceEmpty",
        "FILE_REFERENCE_EXPIRED": "FileReferenceExpired",
        "FILE_REFERENCE_INVALID": "FileReferenceInvalid",
        "FILE_TITLE_EMPTY": "FileTitleEmpty",
        "FILTER_ID_INVALID": "FilterIdInvalid",
        "FILTER_INCLUDE_EMPTY": "FilterIncludeEmpty",
        "FILTER_NOT_SUPPORTED": "FilterNotSupported",
        "FILTER_TITLE_EMPTY": "FilterTitleEmpty",
        "FIRSTNAME_INVALID": "FirstnameInvalid",
        "FOLDER_ID_EMPTY": "FolderIdEmpty",
        "FOLDER_ID_INVALID": "FolderIdInvalid",
        "FRESH_CHANGE_ADMINS_FORBIDDEN": "FreshChangeAdminsForbidden",
        "FROM_MESSAGE_BOT_DISABLED": "FromMessageBotDisabled",
        "FROM_PEER_INVALID": "FromPeerInvalid",
        "GAME_BOT_INVALID": "GameBotInvalid",
        "GEO_POINT_INVALID": "GeoPointInvalid",
        "GIF_CONTENT_TYPE_INVALID": "GifContentTypeInvalid",
        "GIF_ID_INVALID": "GifIdInvalid",
        "GIFT_SLUG_EXPIRED": "GiftSlugExpired",
        "GRAPH_EXPIRED_RELOAD": "GraphExpiredReload",
        "GRAPH_INVALID_RELOAD": "GraphInvalidReload",
        "GRAPH_OUTDATED_RELOAD": "GraphOutdatedReload",
        "GROUPCALL_ALREADY_DISCARDED": "GroupcallAlreadyDiscarded",
        "GROUPCALL_INVALID": "GroupcallInvalid",
        "GROUPCALL_JOIN_MISSING": "GroupcallJoinMissing",
        "GROUPCALL_NOT_MODIFIED": "GroupcallNotModified",
        "GROUPCALL_SSRC_DUPLICATE_MUCH": "GroupcallSsrcDuplicateMuch",
        "GROUPED_MEDIA_INVALID": "GroupedMediaInvalid",
        "GROUP_CALL_INVALID": "GroupCallInvalid",
        "HASH_INVALID": "HashInvalid",
        "HIDE_REQUESTER_MISSING": "HideRequesterMissing",
        "IMAGE_PROCESS_FAILED": "ImageProcessFailed",
        "IMPORT_FILE_INVALID": "ImportFileInvalid",
        "IMPORT_FORMAT_UNRECOGNIZED": "ImportFormatUnrecognized",
        "IMPORT_ID_INVALID": "ImportIdInvalid",
        "INLINE_RESULT_EXPIRED": "InlineResultExpired",
        "INPUT_CONSTRUCTOR_INVALID": "InputConstructorInvalid",
        "INPUT_FETCH_ERROR": "InputFetchError",
        "INPUT_FETCH_FAIL": "InputFetchFail",
        "INPUT_FILTER_INVALID": "InputFilterInvalid",
        "INPUT_LAYER_INVALID": "InputLayerInvalid",
        "INPUT_METHOD_INVALID": "InputMethodInvalid",
        "INPUT_REQUEST_TOO_LONG": "InputRequestTooLong",
        "INPUT_TEXT_EMPTY": "InputTextEmpty",
        "INPUT_USER_DEACTIVATED": "InputUserDeactivated",
        "INVITE_FORBIDDEN_WITH_JOINAS": "InviteForbiddenWithJoinas",
        "INVITE_HASH_EMPTY": "InviteHashEmpty",
        "INVITE_HASH_EXPIRED": "InviteHashExpired",
        "INVITE_HASH_INVALID": "InviteHashInvalid",
        "INVITE_REQUEST_SENT": "InviteRequestSent",
        "INVITE_REVOKED_MISSING": "InviteRevokedMissing",
        "INVITE_SLUG_EMPTY": "InviteSlugEmpty",
        "INVITE_SLUG_EXPIRED": "InviteSlugExpired",
        "INVOICE_PAYLOAD_INVALID": "InvoicePayloadInvalid",
        "JOIN_AS_PEER_INVALID": "JoinAsPeerInvalid",
        "LANG_CODE_INVALID": "LangCodeInvalid",
        "LANG_CODE_NOT_SUPPORTED": "LangCodeNotSupported",
        "LANG_PACK_INVALID": "LangPackInvalid",
        "LASTNAME_INVALID": "LastnameInvalid",
        "LIMIT_INVALID": "LimitInvalid",
        "LINK_NOT_MODIFIED": "LinkNotModified",
        "LOCATION_INVALID": "LocationInvalid",
        "MAX_DATE_INVALID": "MaxDateInvalid",
        "MAX_ID_INVALID": "MaxIdInvalid",
        "MAX_QTS_INVALID": "MaxQtsInvalid",
        "MD5_CHECKSUM_INVALID": "Md5ChecksumInvalid",
        "MEDIA_CAPTION_TOO_LONG": "MediaCaptionTooLong",
        "MEDIA_EMPTY": "MediaEmpty",
        "MEDIA_FILE_INVALID": "MediaFileInvalid",
        "MEDIA_GROUPED_INVALID": "MediaGroupedInvalid",
        "MEDIA_INVALID": "MediaInvalid",
        "MEDIA_NEW_INVALID": "MediaNewInvalid",
        "MEDIA_PREV_INVALID": "MediaPrevInvalid",
        "MEDIA_TTL_INVALID": "MediaTtlInvalid",
        "MEDIA_VIDEO_STORY_MISSING": "MediaVideoStoryMissing",
        "MEGAGROUP_ID_INVALID": "MegagroupIdInvalid",
        "MEGAGROUP_PREHISTORY_HIDDEN": "MegagroupPrehistoryHidden",
        "MEGAGROUP_REQUIRED": "MegagroupRequired",
        "MESSAGE_EDIT_TIME_EXPIRED": "MessageEditTimeExpired",
        "MESSAGE_EMPTY": "MessageEmpty",
        "MESSAGE_IDS_EMPTY": "MessageIdsEmpty",
        "MESSAGE_ID_INVALID": "MessageIdInvalid",
        "MESSAGE_NOT_MODIFIED": "MessageNotModified",
        "MESSAGE_POLL_CLOSED": "MessagePollClosed",
        "MESSAGE_TOO_LONG": "MessageTooLong",
        "METHOD_INVALID": "MethodInvalid",
        "MIN_DATE_INVALID": "MinDateInvalid",
        "MSG_ID_INVALID": "MsgIdInvalid",
        "MSG_TOO_OLD": "MsgTooOld",
        "MSG_VOICE_MISSING": "MsgVoiceMissing",
        "MSG_WAIT_FAILED": "MsgWaitFailed",
        "MULTI_MEDIA_TOO_LONG": "MultiMediaTooLong",
        "NEW_SALT_INVALID": "NewSaltInvalid",
        "NEW_SETTINGS_EMPTY": "NewSettingsEmpty",
        "NEW_SETTINGS_INVALID": "NewSettingsInvalid",
        "NEXT_OFFSET_INVALID": "NextOffsetInvalid",
        "OFFSET_INVALID": "OffsetInvalid",
        "OFFSET_PEER_ID_INVALID": "OffsetPeerIdInvalid",
        "OPTIONS_TOO_MUCH": "OptionsTooMuch",
        "OPTION_INVALID": "OptionInvalid",
        "PACK_SHORT_NAME_INVALID": "PackShortNameInvalid",
        "PACK_SHORT_NAME_OCCUPIED": "PackShortNameOccupied",
        "PACK_TITLE_INVALID": "PackTitleInvalid",
        "PARTICIPANTS_TOO_FEW": "ParticipantsTooFew",
        "PARTICIPANT_ID_INVALID": "ParticipantIdInvalid",
        "PARTICIPANT_JOIN_MISSING": "ParticipantJoinMissing",
        "PARTICIPANT_VERSION_OUTDATED": "ParticipantVersionOutdated",
        "PASSWORD_EMPTY": "PasswordEmpty",
        "PASSWORD_HASH_INVALID": "PasswordHashInvalid",
        "PASSWORD_MISSING": "PasswordMissing",
        "PASSWORD_RECOVERY_NA": "PasswordRecoveryNa",
        "PASSWORD_REQUIRED": "PasswordRequired",
        "PASSWORD_TOO_FRESH_X": "PasswordTooFresh",
        "PAYMENT_PROVIDER_INVALID": "PaymentProviderInvalid",
        "PEER_FLOOD": "PeerFlood",
        "PEER_HISTORY_EMPTY": "PeerHistoryEmpty",
        "PEER_ID_INVALID": "PeerIdInvalid",
        "PEER_ID_NOT_SUPPORTED": "PeerIdNotSupported",
        "PERSISTENT_TIMESTAMP_EMPTY": "PersistentTimestampEmpty",
        "PERSISTENT_TIMESTAMP_INVALID": "PersistentTimestampInvalid",
        "PHONE_CODE_EMPTY": "PhoneCodeEmpty",
        "PHONE_CODE_EXPIRED": "PhoneCodeExpired",
        "PHONE_CODE_HASH_EMPTY": "PhoneCodeHashEmpty",
        "PHONE_CODE_INVALID": "PhoneCodeInvalid",
        "PHONE_HASH_EXPIRED": "PhoneHashExpired",
        "PHONE_NOT_OCCUPIED": "PhoneNotOccupied",
        "PHONE_NUMBER_APP_SIGNUP_FORBIDDEN": "PhoneNumberAppSignupForbidden",
        "PHONE_NUMBER_BANNED": "PhoneNumberBanned",
        "PHONE_NUMBER_FLOOD": "PhoneNumberFlood",
        "PHONE_NUMBER_INVALID": "PhoneNumberInvalid",
        "PHONE_NUMBER_OCCUPIED": "PhoneNumberOccupied",
        "PHONE_NUMBER_UNOCCUPIED": "PhoneNumberUnoccupied",
        "PHONE_PASSWORD_PROTECTED": "PhonePasswordProtected",
        "PHOTO_CONTENT_TYPE_INVALID": "PhotoContentTypeInvalid",
        "PHOTO_CONTENT_URL_EMPTY": "PhotoContentUrlEmpty",
        "PHOTO_CROP_FILE_MISSING": "PhotoCropFileMissing",
        "PHOTO_CROP_SIZE_SMALL": "PhotoCropSizeSmall",
        "PHOTO_EXT_INVALID": "PhotoExtInvalid",
        "PHOTO_FILE_MISSING": "PhotoFileMissing",
        "PHOTO_ID_INVALID": "PhotoIdInvalid",
        "PHOTO_INVALID": "PhotoInvalid",
        "PHOTO_INVALID_DIMENSIONS": "PhotoInvalidDimensions",
        "PHOTO_SAVE_FILE_INVALID": "PhotoSaveFileInvalid",
        "PHOTO_THUMB_URL_EMPTY": "PhotoThumbUrlEmpty",
        "PHOTO_THUMB_URL_INVALID": "PhotoThumbUrlInvalid",
        "PINNED_DIALOGS_TOO_MUCH": "PinnedDialogsTooMuch",
        "PIN_RESTRICTED": "PinRestricted",
        "POLL_ANSWERS_INVALID": "PollAnswersInvalid",
        "POLL_ANSWER_INVALID": "PollAnswerInvalid",
        "POLL_OPTION_DUPLICATE": "PollOptionDuplicate",
        "POLL_OPTION_INVALID": "PollOptionInvalid",
        "POLL_QUESTION_INVALID": "PollQuestionInvalid",
        "POLL_UNSUPPORTED": "PollUnsupported",
        "POLL_VOTE_REQUIRED": "PollVoteRequired",
        "PREMIUM_ACCOUNT_REQUIRED": "PremiumAccountRequired",
        "PRIVACY_KEY_INVALID": "PrivacyKeyInvalid",
        "PRIVACY_TOO_LONG": "PrivacyTooLong",
        "PRIVACY_VALUE_INVALID": "PrivacyValueInvalid",
        "PUBLIC_KEY_REQUIRED": "PublicKeyRequired",
        "QUERY_ID_EMPTY": "QueryIdEmpty",
        "QUERY_ID_INVALID": "QueryIdInvalid",
        "QUERY_TOO_SHORT": "QueryTooShort",
        "QUIZ_ANSWER_MISSING": "QuizAnswerMissing",
        "QUIZ_CORRECT_ANSWERS_EMPTY": "QuizCorrectAnswersEmpty",
        "QUIZ_CORRECT_ANSWERS_TOO_MUCH": "QuizCorrectAnswersTooMuch",
        "QUIZ_CORRECT_ANSWER_INVALID": "QuizCorrectAnswerInvalid",
        "QUIZ_MULTIPLE_INVALID": "QuizMultipleInvalid",
        "RANDOM_ID_EMPTY": "RandomIdEmpty",
        "RANDOM_ID_INVALID": "RandomIdInvalid",
        "RANDOM_LENGTH_INVALID": "RandomLengthInvalid",
        "RANGES_INVALID": "RangesInvalid",
        "REACTION_EMPTY": "ReactionEmpty",
        "REACTION_INVALID": "ReactionInvalid",
        "REFLECTOR_NOT_AVAILABLE": "ReflectorNotAvailable",
        "REPLY_MARKUP_BUY_EMPTY": "ReplyMarkupBuyEmpty",
        "REPLY_MARKUP_GAME_EMPTY": "ReplyMarkupGameEmpty",
        "REPLY_MARKUP_INVALID": "ReplyMarkupInvalid",
        "REPLY_MARKUP_TOO_LONG": "ReplyMarkupTooLong",
        "REPLY_MESSAGE_ID_INVALID": "ReplyMessageIdInvalid",
        "RESET_REQUEST_MISSING": "ResetRequestMissing",
        "RESULTS_TOO_MUCH": "ResultsTooMuch",
        "RESULT_ID_DUPLICATE": "ResultIdDuplicate",
        "RESULT_ID_EMPTY": "ResultIdEmpty",
        "RESULT_ID_INVALID": "ResultIdInvalid",
        "REACTIONS_TOO_MANY": "ReactionsTooMany",
        "RESULT_TYPE_INVALID": "ResultTypeInvalid",
        "REVOTE_NOT_ALLOWED": "RevoteNotAllowed",
        "RIGHTS_NOT_MODIFIED": "RightsNotModified",
        "RSA_DECRYPT_FAILED": "RsaDecryptFailed",
        "SCHEDULE_BOT_NOT_ALLOWED": "ScheduleBotNotAllowed",
        "SCHEDULE_DATE_INVALID": "ScheduleDateInvalid",
        "SCHEDULE_DATE_TOO_LATE": "ScheduleDateTooLate",
        "SCHEDULE_STATUS_PRIVATE": "ScheduleStatusPrivate",
        "SCHEDULE_TOO_MUCH": "ScheduleTooMuch",
        "SCORE_INVALID": "ScoreInvalid",
        "SEARCH_QUERY_EMPTY": "SearchQueryEmpty",
        "SEARCH_WITH_LINK_NOT_SUPPORTED": "SearchWithLinkNotSupported",
        "SECONDS_INVALID": "SecondsInvalid",
        "SEND_AS_PEER_INVALID": "SendAsPeerInvalid",
        "SEND_MESSAGE_MEDIA_INVALID": "SendMessageMediaInvalid",
        "SEND_MESSAGE_TYPE_INVALID": "SendMessageTypeInvalid",
        "SESSION_TOO_FRESH_X": "SessionTooFresh",
        "SETTINGS_INVALID": "SettingsInvalid",
        "SHA256_HASH_INVALID": "Sha256HashInvalid",
        "SHORTNAME_OCCUPY_FAILED": "ShortnameOccupyFailed",
        "SHORT_NAME_INVALID": "ShortNameInvalid",
        "SHORT_NAME_OCCUPIED": "ShortNameOccupied",
        "SLOWMODE_MULTI_MSGS_DISABLED": "SlowmodeMultiMsgsDisabled",
        "SMS_CODE_CREATE_FAILED": "SmsCodeCreateFailed",
        "SRP_ID_INVALID": "SrpIdInvalid",
        "SRP_PASSWORD_CHANGED": "SrpPasswordChanged",
        "START_PARAM_EMPTY": "StartParamEmpty",
        "START_PARAM_INVALID": "StartParamInvalid",
        "START_PARAM_TOO_LONG": "StartParamTooLong",
        "STICKERPACK_STICKERS_TOO_MUCH": "StickerpackStickersTooMuch",
        "STICKERSET_INVALID": "StickersetInvalid",
        "STICKERSET_NOT_MODIFIED": "StickersetNotModified",
        "STICKERS_EMPTY": "StickersEmpty",
        "STICKERS_TOO_MUCH": "StickersTooMuch",
        "STICKER_DOCUMENT_INVALID": "StickerDocumentInvalid",
        "STICKER_EMOJI_INVALID": "StickerEmojiInvalid",
        "STICKER_FILE_INVALID": "StickerFileInvalid",
        "STICKER_GIF_DIMENSIONS": "StickerGifDimensions",
        "STICKER_ID_INVALID": "StickerIdInvalid",
        "STICKER_INVALID": "StickerInvalid",
        "STICKER_MIME_INVALID": "StickerMimeInvalid",
        "STICKER_PNG_DIMENSIONS": "StickerPngDimensions",
        "STICKER_PNG_NOPNG": "StickerPngNopng",
        "STICKER_TGS_NODOC": "StickerTgsNodoc",
        "STICKER_TGS_NOTGS": "StickerTgsNotgs",
        "STICKER_THUMB_PNG_NOPNG": "StickerThumbPngNopng",
        "STICKER_VIDEO_BIG": "StickerVideoBig",
        "STICKER_VIDEO_NODOC": "StickerVideoNodoc",
        "STICKER_VIDEO_NOWEBM": "StickerVideoNowebm",
        "STORIES_TOO_MUCH": "StoriesTooMuch",
        "STORY_PERIOD_INVALID": "StoryPeriodInvalid",
        "SWITCH_PM_TEXT_EMPTY": "SwitchPmTextEmpty",
        "TAKEOUT_INVALID": "TakeoutInvalid",
        "TAKEOUT_REQUIRED": "TakeoutRequired",
        "TEMP_AUTH_KEY_ALREADY_BOUND": "TempAuthKeyAlreadyBound",
        "TEMP_AUTH_KEY_EMPTY": "TempAuthKeyEmpty",
        "THEME_FILE_INVALID": "ThemeFileInvalid",
        "THEME_FORMAT_INVALID": "ThemeFormatInvalid",
        "THEME_INVALID": "ThemeInvalid",
        "THEME_MIME_INVALID": "ThemeMimeInvalid",
        "THEME_TITLE_INVALID": "ThemeTitleInvalid",
        "TITLE_INVALID": "TitleInvalid",
        "TMP_PASSWORD_DISABLED": "TmpPasswordDisabled",
        "TMP_PASSWORD_INVALID": "TmpPasswordInvalid",
        "TOKEN_INVALID": "TokenInvalid",
        "TOPIC_CLOSED": "TopicClosed",
        "TOPIC_DELETED": "TopicDeleted",
        "TOPIC_NOT_MODIFIED": "TopicNotModified",
        "TO_LANG_INVALID": "ToLangInvalid",
        "TRANSCRIPTION_FAILED": "TranscriptionFailed",
        "TTL_DAYS_INVALID": "TtlDaysInvalid",
        "TTL_MEDIA_INVALID": "TtlMediaInvalid",
        "TYPES_EMPTY": "TypesEmpty",
        "TYPE_CONSTRUCTOR_INVALID": "TypeConstructorInvalid",
        "UNKNOWN_ERROR": "UnknownError",
        "UNTIL_DATE_INVALID": "UntilDateInvalid",
        "URL_INVALID": "UrlInvalid",
        "USAGE_LIMIT_INVALID": "UsageLimitInvalid",
        "USERNAME_INVALID": "UsernameInvalid",
        "USERNAME_NOT_MODIFIED": "UsernameNotModified",
        "USERNAME_NOT_OCCUPIED": "UsernameNotOccupied",
        "USERNAME_OCCUPIED": "UsernameOccupied",
        "USERNAME_PURCHASE_AVAILABLE": "UsernamePurchaseAvailable",
        "USERPIC_UPLOAD_REQUIRED": "UserpicUploadRequired",
        "USERS_TOO_FEW": "UsersTooFew",
        "USERS_TOO_MUCH": "UsersTooMuch",
        "USER_ADMIN_INVALID": "UserAdminInvalid",
        "USER_ALREADY_INVITED": "UserAlreadyInvited",
        "USER_ALREADY_PARTICIPANT": "UserAlreadyParticipant",
        "USER_BANNED_IN_CHANNEL": "UserBannedInChannel",
        "USER_BLOCKED": "UserBlocked",
        "USER_BOT": "UserBot",
        "USER_BOT_INVALID": "UserBotInvalid",
        "USER_BOT_REQUIRED": "UserBotRequired",
        "USER_CHANNELS_TOO_MUCH": "UserChannelsTooMuch",
        "USER_CREATOR": "UserCreator",
        "USER_ID_INVALID": "UserIdInvalid",
        "USER_INVALID": "UserInvalid",
        "USER_IS_BLOCKED": "UserIsBlocked",
        "USER_IS_BOT": "UserIsBot",
        "USER_KICKED": "UserKicked",
        "USER_NOT_MUTUAL_CONTACT": "UserNotMutualContact",
        "USER_NOT_PARTICIPANT": "UserNotParticipant",
        "USER_PUBLIC_MISSING": "UserPublicMissing",
        "USER_VOLUME_INVALID": "UserVolumeInvalid",
        "VIDEO_CONTENT_TYPE_INVALID": "VideoContentTypeInvalid",
        "VIDEO_FILE_INVALID": "VideoFileInvalid",
        "VIDEO_TITLE_EMPTY": "VideoTitleEmpty",
        "VOICE_MESSAGES_FORBIDDEN": "VoiceMessagesForbidden",
        "VOLUME_LOC_NOT_FOUND": "VolumeLocNotFound",
        "WALLPAPER_FILE_INVALID": "WallpaperFileInvalid",
        "WALLPAPER_INVALID": "WallpaperInvalid",
        "WALLPAPER_MIME_INVALID": "WallpaperMimeInvalid",
        "WC_CONVERT_URL_INVALID": "WcConvertUrlInvalid",
        "WEBDOCUMENT_INVALID": "WebdocumentInvalid",
        "WEBDOCUMENT_MIME_INVALID": "WebdocumentMimeInvalid",
        "WEBDOCUMENT_SIZE_TOO_BIG": "WebdocumentSizeTooBig",
        "WEBDOCUMENT_URL_EMPTY": "WebdocumentUrlEmpty",
        "WEBDOCUMENT_URL_INVALID": "WebdocumentUrlInvalid",
        "WEBPAGE_CURL_FAILED": "WebpageCurlFailed",
        "WEBPAGE_MEDIA_EMPTY": "WebpageMediaEmpty",
        "WEBPAGE_NOT_FOUND": "WebpageNotFound",
        "WEBPAGE_URL_INVALID": "WebpageUrlInvalid",
        "WEBPUSH_AUTH_INVALID": "WebpushAuthInvalid",
        "WEBPUSH_KEY_INVALID": "WebpushKeyInvalid",
        "WEBPUSH_TOKEN_INVALID": "WebpushTokenInvalid",
        "YOU_BLOCKED_USER": "YouBlockedUser",
        "STORIES_NEVER_CREATED": "StoriesNeverCreated",
        "MEDIA_FILE_INVALID": "MediaFileInvalid",
        "CHANNEL_FORUM_MISSING": "ChannelForumMissing",
        "TTL_PERIOD_INVALID": "TtlPeriodInvalid",
        "BOOSTS_REQUIRED": "BoostsRequired",
    },
    500: {
        "_": "InternalServerError",
        "API_CALL_ERROR": "ApiCallError",
        "AUTH_RESTART": "AuthRestart",
        "CALL_OCCUPY_FAILED": "CallOccupyFailed",
        "CHAT_ID_GENERATE_FAILED": "ChatIdGenerateFailed",
        "CHAT_OCCUPY_LOC_FAILED": "ChatOccupyLocFailed",
        "CHAT_OCCUPY_USERNAME_FAILED": "ChatOccupyUsernameFailed",
        "CHP_CALL_FAIL": "ChpCallFail",
        "ENCRYPTION_OCCUPY_ADMIN_FAILED": "EncryptionOccupyAdminFailed",
        "ENCRYPTION_OCCUPY_FAILED": "EncryptionOccupyFailed",
        "FOLDER_DEAC_AUTOFIX_ALL": "FolderDeacAutofixAll",
        "GROUPCALL_ADD_PARTICIPANTS_FAILED": "GroupcallAddParticipantsFailed",
        "GROUPED_ID_OCCUPY_FAILED": "GroupedIdOccupyFailed",
        "HISTORY_GET_FAILED": "HistoryGetFailed",
        "IMAGE_ENGINE_DOWN": "ImageEngineDown",
        "INTERDC_X_CALL_ERROR": "InterdcCallError",
        "INTERDC_X_CALL_RICH_ERROR": "InterdcCallRichError",
        "MEMBER_FETCH_FAILED": "MemberFetchFailed",
        "MEMBER_NO_LOCATION": "MemberNoLocation",
        "MEMBER_OCCUPY_PRIMARY_LOC_FAILED": "MemberOccupyPrimaryLocFailed",
        "MEMBER_OCCUPY_USERNAME_FAILED": "MemberOccupyUsernameFailed",
        "MSGID_DECREASE_RETRY": "MsgidDecreaseRetry",
        "MSG_RANGE_UNSYNC": "MsgRangeUnsync",
        "MT_SEND_QUEUE_TOO_LONG": "MtSendQueueTooLong",
        "NEED_CHAT_INVALID": "NeedChatInvalid",
        "NEED_MEMBER_INVALID": "NeedMemberInvalid",
        "No workers running": "NoWorkersRunning",
        "PARTICIPANT_CALL_FAILED": "ParticipantCallFailed",
        "PERSISTENT_TIMESTAMP_OUTDATED": "PersistentTimestampOutdated",
        "PHOTO_CREATE_FAILED": "PhotoCreateFailed",
        "POSTPONED_TIMEOUT": "PostponedTimeout",
        "PTS_CHANGE_EMPTY": "PtsChangeEmpty",
        "RANDOM_ID_DUPLICATE": "RandomIdDuplicate",
        "REG_ID_GENERATE_FAILED": "RegIdGenerateFailed",
        "RPC_CALL_FAIL": "RpcCallFail",
        "RPC_CONNECT_FAILED": "RpcConnectFailed",
        "RPC_MCGET_FAIL": "RpcMcgetFail",
        "SIGN_IN_FAILED": "SignInFailed",
        "STORAGE_CHECK_FAILED": "StorageCheckFailed",
        "STORE_INVALID_SCALAR_TYPE": "StoreInvalidScalarType",
        "TIMEOUT": "Timeout",
        "UNKNOWN_METHOD": "UnknownMethod",
        "UPLOAD_NO_VOLUME": "UploadNoVolume",
        "VOLUME_LOC_NOT_FOUND": "VolumeLocNotFound",
        "WORKER_BUSY_TOO_LONG_RETRY": "WorkerBusyTooLongRetry",
        "WP_ID_GENERATE_FAILED": "WpIdGenerateFailed",
        "FILE_WRITE_FAILED": "FileWriteFailed",
        "CHAT_FROM_CALL_CHANGED": "ChatFromCallChanged",
    },
}



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class SentCodeTypeSetUpEmailRequired(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.auth.SentCodeType`.

    Details:
        - Layer: ``177``
        - ID: ``A5491DEA``

    Parameters:
        apple_signin_allowed (``bool``, *optional*):
            N/A

        google_signin_allowed (``bool``, *optional*):
            N/A

    """

    __slots__: List[str] = ["apple_signin_allowed", "google_signin_allowed"]

    ID = 0xa5491dea
    QUALNAME = "types.auth.SentCodeTypeSetUpEmailRequired"

    def __init__(self, *, apple_signin_allowed: Optional[bool] = None, google_signin_allowed: Optional[bool] = None) -> None:
        self.apple_signin_allowed = apple_signin_allowed  # flags.0?true
        self.google_signin_allowed = google_signin_allowed  # flags.1?true

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "SentCodeTypeSetUpEmailRequired":
        
        flags = Int.read(b)
        
        apple_signin_allowed = True if flags & (1 << 0) else False
        google_signin_allowed = True if flags & (1 << 1) else False
        return SentCodeTypeSetUpEmailRequired(apple_signin_allowed=apple_signin_allowed, google_signin_allowed=google_signin_allowed)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.apple_signin_allowed else 0
        flags |= (1 << 1) if self.google_signin_allowed else 0
        b.write(Int(flags))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class ExportMessageLink(TLObject):  # type: ignore
    """Telegram API function.

    Details:
        - Layer: ``177``
        - ID: ``E63FADEB``

    Parameters:
        channel (:obj:`InputChannel <pyrogram.raw.base.InputChannel>`):
            N/A

        id (``int`` ``32-bit``):
            N/A

        grouped (``bool``, *optional*):
            N/A

        thread (``bool``, *optional*):
            N/A

    Returns:
        :obj:`ExportedMessageLink <pyrogram.raw.base.ExportedMessageLink>`
    """

    __slots__: List[str] = ["channel", "id", "grouped", "thread"]

    ID = 0xe63fadeb
    QUALNAME = "functions.channels.ExportMessageLink"

    def __init__(self, *, channel: "raw.base.InputChannel", id: int, grouped: Optional[bool] = None, thread: Optional[bool] = None) -> None:
        self.channel = channel  # InputChannel
        self.id = id  # int
        self.grouped = grouped  # flags.0?true
        self.thread = thread  # flags.1?true

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "ExportMessageLink":
        
        flags = Int.read(b)
        
        grouped = True if flags & (1 << 0) else False
        thread = True if flags & (1 << 1) else False
        channel = TLObject.read(b)
        
        id = Int.read(b)
        
        return ExportMessageLink(channel=channel, id=id, grouped=grouped, thread=thread)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.grouped else 0
        flags |= (1 << 1) if self.thread else 0
        b.write(Int(flags))
        
        b.write(self.channel.write())
        
        b.write(Int(self.id))
        
        return b.getvalue()



# Pyrofork - Telegram MTProto API Client Library for Python
# Copyright (C) 2017-present Dan <https://github.com/delivrance>
# Copyright (C) 2022-present Mayuri-Chan <https://github.com/Mayuri-Chan>
#
# This file is part of Pyrofork.
#
# Pyrofork is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Pyrofork is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
#ᴘʏʀᴏɢᴇʀᴀᴍ If not, see <http://www.gnu.org/licenses/>.

from ..rpc_error import RPCError


class Flood(RPCError):
    """Flood"""
    CODE = 420
    """``int``: RPC Error Code"""
    NAME = __doc__


class TwoFaConfirmWait(Flood):
    """A wait of {value} seconds is required because this account is active and protected by a 2FA password"""
    ID = "2FA_CONFIRM_WAIT_X"
    """``str``: RPC Error ID"""
    MESSAGE = __doc__


class FloodTestPhoneWait(Flood):
    """A wait of {value} seconds is required in the test servers"""
    ID = "FLOOD_TEST_PHONE_WAIT_X"
    """``str``: RPC Error ID"""
    MESSAGE = __doc__


class FloodWait(Flood):
    """A wait of {value} seconds is required"""
    ID = "FLOOD_WAIT_X"
    """``str``: RPC Error ID"""
    MESSAGE = __doc__


class PremiumSubActiveUntil(Flood):
    """A wait of {value} seconds is required"""
    ID = "PREMIUM_SUB_ACTIVE_UNTIL_X"
    """``str``: RPC Error ID"""
    MESSAGE = __doc__


class SlowmodeWait(Flood):
    """A wait of {value} seconds is required to send messages in this chat"""
    ID = "SLOWMODE_WAIT_X"
    """``str``: RPC Error ID"""
    MESSAGE = __doc__


class StorySendFlood(Flood):
    """A wait of {value} seconds is required to continue posting stories"""
    ID = "STORY_SEND_FLOOD_X"
    """``str``: RPC Error ID"""
    MESSAGE = __doc__


class TakeoutInitDelay(Flood):
    """You have to confirm the data export request using one of your mobile devices or wait {value} seconds"""
    ID = "TAKEOUT_INIT_DELAY_X"
    """``str``: RPC Error ID"""
    MESSAGE = __doc__





#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from pyrogram import raw
from typing import List
from ..object import Object

class ExportedStoryLink(Object):
    """Contains information about a story viewers.


    Parameters:
        link (``str``):
            The link of the story.
    """

    def __init__(
            self, *,
            link: str
    ):
        super().__init__()

        self.link = link

    @staticmethod
    def _parse(exportedstorylink: "raw.types.ExportedStoryLink") -> "ExportedStoryLink":
        return ExportedStoryLink(
            link=getattr(exportedstorylink,"link", None)
        )



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from typing import Union

import pyrogram
from pyrogram import raw, types


class GetChatAdminsWithInviteLinks:
    async def get_chat_admins_with_invite_links(
        self: "pyrogram.Client",
        chat_id: Union[int, str],
    ):
        """Get the list of the administrators that have exported invite links in a chat.

        You must be the owner of a chat for this to work.

        .. include:: /_includes/usable-by/users.rst

        Parameters:
            chat_id (``int`` | ``str``):
                Unique identifier for the target chat or username of the target channel/supergroup
                (in the format @username).
                You can also use chat public link in form of *t.me/<username>* (str).

        Returns:
            List of :obj:`~pyrogram.types.ChatAdminWithInviteLink`: On success, the list of admins that have exported
            invite links is returned.
        """
        r = await self.invoke(
            raw.functions.messages.GetAdminsWithInvites(
                peer=await self.resolve_peer(chat_id)
            )
        )

        users = {i.id: i for i in r.users}

        return types.List(
            types.ChatAdminWithInviteLinks._parse(self, admin, users)
            for admin in r.admins
        )



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class ConfigSimple(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.help.ConfigSimple`.

    Details:
        - Layer: ``177``
        - ID: ``5A592A6C``

    Parameters:
        date (``int`` ``32-bit``):
            N/A

        expires (``int`` ``32-bit``):
            N/A

        rules (List of :obj:`AccessPointRule <pyrogram.raw.base.AccessPointRule>`):
            N/A

    """

    __slots__: List[str] = ["date", "expires", "rules"]

    ID = 0x5a592a6c
    QUALNAME = "types.help.ConfigSimple"

    def __init__(self, *, date: int, expires: int, rules: List["raw.base.AccessPointRule"]) -> None:
        self.date = date  # int
        self.expires = expires  # int
        self.rules = rules  # vector<AccessPointRule>

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "ConfigSimple":
        # No flags
        
        date = Int.read(b)
        
        expires = Int.read(b)
        
        rules = TLObject.read(b)
        
        return ConfigSimple(date=date, expires=expires, rules=rules)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        # No flags
        
        b.write(Int(self.date))
        
        b.write(Int(self.expires))
        
        b.write(Vector(self.rules))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from typing import List

import pyrogram
from pyrogram import raw
from pyrogram import types
from ..object import Object


class InlineKeyboardMarkup(Object):
    """An inline keyboard that appears right next to the message it belongs to.

    Parameters:
        inline_keyboard (List of List of :obj:`~pyrogram.types.InlineKeyboardButton`):
            List of button rows, each represented by a List of InlineKeyboardButton objects.
    """

    def __init__(self, inline_keyboard: List[List["types.InlineKeyboardButton"]]):
        super().__init__()

        self.inline_keyboard = inline_keyboard

    @staticmethod
    def read(o):
        inline_keyboard = []

        for i in o.rows:
            row = []

            for j in i.buttons:
                row.append(types.InlineKeyboardButton.read(j))

            inline_keyboard.append(row)

        return InlineKeyboardMarkup(
            inline_keyboard=inline_keyboard
        )

    async def write(self, client: "pyrogram.Client"):
        rows = []

        for r in self.inline_keyboard:
            buttons = []

            for b in r:
                buttons.append(await b.write(client))

            rows.append(raw.types.KeyboardButtonRow(buttons=buttons))

        return raw.types.ReplyInlineMarkup(rows=rows)

        # There seems to be a Python issues with nested async comprehensions.
        # See: https://bugs.python.org/issue33346
        #
        # return raw.types.ReplyInlineMarkup(
        #     rows=[raw.types.KeyboardButtonRow(
        #         buttons=[await j.write(client) for j in i]
        #     ) for i in self.inline_keyboard]
        # )



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class DraftMessage(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.DraftMessage`.

    Details:
        - Layer: ``177``
        - ID: ``3FCCF7EF``

    Parameters:
        message (``str``):
            N/A

        date (``int`` ``32-bit``):
            N/A

        no_webpage (``bool``, *optional*):
            N/A

        invert_media (``bool``, *optional*):
            N/A

        reply_to (:obj:`InputReplyTo <pyrogram.raw.base.InputReplyTo>`, *optional*):
            N/A

        entities (List of :obj:`MessageEntity <pyrogram.raw.base.MessageEntity>`, *optional*):
            N/A

        media (:obj:`InputMedia <pyrogram.raw.base.InputMedia>`, *optional*):
            N/A

    """

    __slots__: List[str] = ["message", "date", "no_webpage", "invert_media", "reply_to", "entities", "media"]

    ID = 0x3fccf7ef
    QUALNAME = "types.DraftMessage"

    def __init__(self, *, message: str, date: int, no_webpage: Optional[bool] = None, invert_media: Optional[bool] = None, reply_to: "raw.base.InputReplyTo" = None, entities: Optional[List["raw.base.MessageEntity"]] = None, media: "raw.base.InputMedia" = None) -> None:
        self.message = message  # string
        self.date = date  # int
        self.no_webpage = no_webpage  # flags.1?true
        self.invert_media = invert_media  # flags.6?true
        self.reply_to = reply_to  # flags.4?InputReplyTo
        self.entities = entities  # flags.3?Vector<MessageEntity>
        self.media = media  # flags.5?InputMedia

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "DraftMessage":
        
        flags = Int.read(b)
        
        no_webpage = True if flags & (1 << 1) else False
        invert_media = True if flags & (1 << 6) else False
        reply_to = TLObject.read(b) if flags & (1 << 4) else None
        
        message = String.read(b)
        
        entities = TLObject.read(b) if flags & (1 << 3) else []
        
        media = TLObject.read(b) if flags & (1 << 5) else None
        
        date = Int.read(b)
        
        return DraftMessage(message=message, date=date, no_webpage=no_webpage, invert_media=invert_media, reply_to=reply_to, entities=entities, media=media)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 1) if self.no_webpage else 0
        flags |= (1 << 6) if self.invert_media else 0
        flags |= (1 << 4) if self.reply_to is not None else 0
        flags |= (1 << 3) if self.entities else 0
        flags |= (1 << 5) if self.media is not None else 0
        b.write(Int(flags))
        
        if self.reply_to is not None:
            b.write(self.reply_to.write())
        
        b.write(String(self.message))
        
        if self.entities is not None:
            b.write(Vector(self.entities))
        
        if self.media is not None:
            b.write(self.media.write())
        
        b.write(Int(self.date))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

import inspect
from typing import Callable

import pyrogram
from pyrogram.filters import Filter
from pyrogram.types import Update


class Handler:
    def __init__(self, callback: Callable, filters: Filter = None):
        self.callback = callback
        self.filters = filters

    async def check(self, client: "pyrogram.Client", update: Update):
        if callable(self.filters):
            if inspect.iscoroutinefunction(self.filters.__call__):
                return await self.filters(client, update)
            else:
                return await client.loop.run_in_executor(
                    client.executor,
                    self.filters,
                    client, update
                )

        return True



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

import pyrogram
from pyrogram import raw
from .bot_command_scope import BotCommandScope


class BotCommandScopeAllGroupChats(BotCommandScope):
    """Represents the scope of bot commands, covering all group and supergroup chats.
    """

    def __init__(self):
        super().__init__("all_group_chats")

    async def write(self, client: "pyrogram.Client") -> "raw.base.BotCommandScope":
        return raw.types.BotCommandScopeChats()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class AttachMenuBot(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.AttachMenuBot`.

    Details:
        - Layer: ``177``
        - ID: ``D90D8DFE``

    Parameters:
        bot_id (``int`` ``64-bit``):
            N/A

        short_name (``str``):
            N/A

        icons (List of :obj:`AttachMenuBotIcon <pyrogram.raw.base.AttachMenuBotIcon>`):
            N/A

        inactive (``bool``, *optional*):
            N/A

        has_settings (``bool``, *optional*):
            N/A

        request_write_access (``bool``, *optional*):
            N/A

        show_in_attach_menu (``bool``, *optional*):
            N/A

        show_in_side_menu (``bool``, *optional*):
            N/A

        side_menu_disclaimer_needed (``bool``, *optional*):
            N/A

        peer_types (List of :obj:`AttachMenuPeerType <pyrogram.raw.base.AttachMenuPeerType>`, *optional*):
            N/A

    """

    __slots__: List[str] = ["bot_id", "short_name", "icons", "inactive", "has_settings", "request_write_access", "show_in_attach_menu", "show_in_side_menu", "side_menu_disclaimer_needed", "peer_types"]

    ID = 0xd90d8dfe
    QUALNAME = "types.AttachMenuBot"

    def __init__(self, *, bot_id: int, short_name: str, icons: List["raw.base.AttachMenuBotIcon"], inactive: Optional[bool] = None, has_settings: Optional[bool] = None, request_write_access: Optional[bool] = None, show_in_attach_menu: Optional[bool] = None, show_in_side_menu: Optional[bool] = None, side_menu_disclaimer_needed: Optional[bool] = None, peer_types: Optional[List["raw.base.AttachMenuPeerType"]] = None) -> None:
        self.bot_id = bot_id  # long
        self.short_name = short_name  # string
        self.icons = icons  # Vector<AttachMenuBotIcon>
        self.inactive = inactive  # flags.0?true
        self.has_settings = has_settings  # flags.1?true
        self.request_write_access = request_write_access  # flags.2?true
        self.show_in_attach_menu = show_in_attach_menu  # flags.3?true
        self.show_in_side_menu = show_in_side_menu  # flags.4?true
        self.side_menu_disclaimer_needed = side_menu_disclaimer_needed  # flags.5?true
        self.peer_types = peer_types  # flags.3?Vector<AttachMenuPeerType>

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "AttachMenuBot":
        
        flags = Int.read(b)
        
        inactive = True if flags & (1 << 0) else False
        has_settings = True if flags & (1 << 1) else False
        request_write_access = True if flags & (1 << 2) else False
        show_in_attach_menu = True if flags & (1 << 3) else False
        show_in_side_menu = True if flags & (1 << 4) else False
        side_menu_disclaimer_needed = True if flags & (1 << 5) else False
        bot_id = Long.read(b)
        
        short_name = String.read(b)
        
        peer_types = TLObject.read(b) if flags & (1 << 3) else []
        
        icons = TLObject.read(b)
        
        return AttachMenuBot(bot_id=bot_id, short_name=short_name, icons=icons, inactive=inactive, has_settings=has_settings, request_write_access=request_write_access, show_in_attach_menu=show_in_attach_menu, show_in_side_menu=show_in_side_menu, side_menu_disclaimer_needed=side_menu_disclaimer_needed, peer_types=peer_types)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.inactive else 0
        flags |= (1 << 1) if self.has_settings else 0
        flags |= (1 << 2) if self.request_write_access else 0
        flags |= (1 << 3) if self.show_in_attach_menu else 0
        flags |= (1 << 4) if self.show_in_side_menu else 0
        flags |= (1 << 5) if self.side_menu_disclaimer_needed else 0
        flags |= (1 << 3) if self.peer_types else 0
        b.write(Int(flags))
        
        b.write(Long(self.bot_id))
        
        b.write(String(self.short_name))
        
        if self.peer_types is not None:
            b.write(Vector(self.peer_types))
        
        b.write(Vector(self.icons))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from .bool import Bool, BoolFalse, BoolTrue
from .bytes import Bytes
from .double import Double
from .int import Int, Long, Int128, Int256
from .string import String
from .vector import Vector



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from .add_sticker_to_set import AddStickerToSet
from .create_sticker_set import CreateStickerSet
from .get_sticker_set import GetStickerSet

class Stickers(
    AddStickerToSet,
    CreateStickerSet,
    GetStickerSet
):
    pass



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class GetAuthorizationForm(TLObject):  # type: ignore
    """Telegram API function.

    Details:
        - Layer: ``177``
        - ID: ``A929597A``

    Parameters:
        bot_id (``int`` ``64-bit``):
            N/A

        scope (``str``):
            N/A

        public_key (``str``):
            N/A

    Returns:
        :obj:`account.AuthorizationForm <pyrogram.raw.base.account.AuthorizationForm>`
    """

    __slots__: List[str] = ["bot_id", "scope", "public_key"]

    ID = 0xa929597a
    QUALNAME = "functions.account.GetAuthorizationForm"

    def __init__(self, *, bot_id: int, scope: str, public_key: str) -> None:
        self.bot_id = bot_id  # long
        self.scope = scope  # string
        self.public_key = public_key  # string

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "GetAuthorizationForm":
        # No flags
        
        bot_id = Long.read(b)
        
        scope = String.read(b)
        
        public_key = String.read(b)
        
        return GetAuthorizationForm(bot_id=bot_id, scope=scope, public_key=public_key)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        # No flags
        
        b.write(Long(self.bot_id))
        
        b.write(String(self.scope))
        
        b.write(String(self.public_key))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from .change_cloud_password import ChangeCloudPassword
from .enable_cloud_password import EnableCloudPassword
from .remove_cloud_password import RemoveCloudPassword


class Password(
    RemoveCloudPassword,
    ChangeCloudPassword,
    EnableCloudPassword
):
    pass



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

import pyrogram
from pyrogram import raw


class GetContactsCount:
    async def get_contacts_count(
        self: "pyrogram.Client"
    ) -> int:
        """Get the total count of contacts from your Telegram address book.

        .. include:: /_includes/usable-by/users.rst

        Returns:
            ``int``: On success, the contacts count is returned.

        Example:
            .. code-block:: python

                count = await app.get_contacts_count()
                print(count)
        """

        return len((await self.invoke(raw.functions.contacts.GetContacts(hash=0))).contacts)



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class UpdateChannelPinnedTopics(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.Update`.

    Details:
        - Layer: ``177``
        - ID: ``FE198602``

    Parameters:
        channel_id (``int`` ``64-bit``):
            N/A

        order (List of ``int`` ``32-bit``, *optional*):
            N/A

    """

    __slots__: List[str] = ["channel_id", "order"]

    ID = 0xfe198602
    QUALNAME = "types.UpdateChannelPinnedTopics"

    def __init__(self, *, channel_id: int, order: Optional[List[int]] = None) -> None:
        self.channel_id = channel_id  # long
        self.order = order  # flags.0?Vector<int>

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "UpdateChannelPinnedTopics":
        
        flags = Int.read(b)
        
        channel_id = Long.read(b)
        
        order = TLObject.read(b, Int) if flags & (1 << 0) else []
        
        return UpdateChannelPinnedTopics(channel_id=channel_id, order=order)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.order else 0
        b.write(Int(flags))
        
        b.write(Long(self.channel_id))
        
        if self.order is not None:
            b.write(Vector(self.order, Int))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class PhotoCachedSize(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.PhotoSize`.

    Details:
        - Layer: ``177``
        - ID: ``21E1AD6``

    Parameters:
        type (``str``):
            N/A

        w (``int`` ``32-bit``):
            N/A

        h (``int`` ``32-bit``):
            N/A

        bytes (``bytes``):
            N/A

    """

    __slots__: List[str] = ["type", "w", "h", "bytes"]

    ID = 0x21e1ad6
    QUALNAME = "types.PhotoCachedSize"

    def __init__(self, *, type: str, w: int, h: int, bytes: bytes) -> None:
        self.type = type  # string
        self.w = w  # int
        self.h = h  # int
        self.bytes = bytes  # bytes

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "PhotoCachedSize":
        # No flags
        
        type = String.read(b)
        
        w = Int.read(b)
        
        h = Int.read(b)
        
        bytes = Bytes.read(b)
        
        return PhotoCachedSize(type=type, w=w, h=h, bytes=bytes)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        # No flags
        
        b.write(String(self.type))
        
        b.write(Int(self.w))
        
        b.write(Int(self.h))
        
        b.write(Bytes(self.bytes))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class AllStoriesNotModified(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.stories.AllStories`.

    Details:
        - Layer: ``177``
        - ID: ``1158FE3E``

    Parameters:
        state (``str``):
            N/A

        stealth_mode (:obj:`StoriesStealthMode <pyrogram.raw.base.StoriesStealthMode>`):
            N/A

    Functions:
        This object can be returned by 1 function.

        .. currentmodule:: pyrogram.raw.functions

        .. autosummary::
            :nosignatures:

            stories.GetAllStories
    """

    __slots__: List[str] = ["state", "stealth_mode"]

    ID = 0x1158fe3e
    QUALNAME = "types.stories.AllStoriesNotModified"

    def __init__(self, *, state: str, stealth_mode: "raw.base.StoriesStealthMode") -> None:
        self.state = state  # string
        self.stealth_mode = stealth_mode  # StoriesStealthMode

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "AllStoriesNotModified":
        
        flags = Int.read(b)
        
        state = String.read(b)
        
        stealth_mode = TLObject.read(b)
        
        return AllStoriesNotModified(state=state, stealth_mode=stealth_mode)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        
        b.write(Int(flags))
        
        b.write(String(self.state))
        
        b.write(self.stealth_mode.write())
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #

from .save_file_part import SaveFilePart
from .get_file import GetFile
from .save_big_file_part import SaveBigFilePart
from .get_web_file import GetWebFile
from .get_cdn_file import GetCdnFile
from .reupload_cdn_file import ReuploadCdnFile
from .get_cdn_file_hashes import GetCdnFileHashes
from .get_file_hashes import GetFileHashes



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class ChannelAdminLogEventActionExportedInviteEdit(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.ChannelAdminLogEventAction`.

    Details:
        - Layer: ``177``
        - ID: ``E90EBB59``

    Parameters:
        prev_invite (:obj:`ExportedChatInvite <pyrogram.raw.base.ExportedChatInvite>`):
            N/A

        new_invite (:obj:`ExportedChatInvite <pyrogram.raw.base.ExportedChatInvite>`):
            N/A

    """

    __slots__: List[str] = ["prev_invite", "new_invite"]

    ID = 0xe90ebb59
    QUALNAME = "types.ChannelAdminLogEventActionExportedInviteEdit"

    def __init__(self, *, prev_invite: "raw.base.ExportedChatInvite", new_invite: "raw.base.ExportedChatInvite") -> None:
        self.prev_invite = prev_invite  # ExportedChatInvite
        self.new_invite = new_invite  # ExportedChatInvite

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "ChannelAdminLogEventActionExportedInviteEdit":
        # No flags
        
        prev_invite = TLObject.read(b)
        
        new_invite = TLObject.read(b)
        
        return ChannelAdminLogEventActionExportedInviteEdit(prev_invite=prev_invite, new_invite=new_invite)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        # No flags
        
        b.write(self.prev_invite.write())
        
        b.write(self.new_invite.write())
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class BroadcastRevenueTransactionWithdrawal(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.BroadcastRevenueTransaction`.

    Details:
        - Layer: ``177``
        - ID: ``5A590978``

    Parameters:
        amount (``int`` ``64-bit``):
            N/A

        date (``int`` ``32-bit``):
            N/A

        provider (``str``):
            N/A

        pending (``bool``, *optional*):
            N/A

        failed (``bool``, *optional*):
            N/A

        transaction_date (``int`` ``32-bit``, *optional*):
            N/A

        transaction_url (``str``, *optional*):
            N/A

    """

    __slots__: List[str] = ["amount", "date", "provider", "pending", "failed", "transaction_date", "transaction_url"]

    ID = 0x5a590978
    QUALNAME = "types.BroadcastRevenueTransactionWithdrawal"

    def __init__(self, *, amount: int, date: int, provider: str, pending: Optional[bool] = None, failed: Optional[bool] = None, transaction_date: Optional[int] = None, transaction_url: Optional[str] = None) -> None:
        self.amount = amount  # long
        self.date = date  # int
        self.provider = provider  # string
        self.pending = pending  # flags.0?true
        self.failed = failed  # flags.2?true
        self.transaction_date = transaction_date  # flags.1?int
        self.transaction_url = transaction_url  # flags.1?string

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "BroadcastRevenueTransactionWithdrawal":
        
        flags = Int.read(b)
        
        pending = True if flags & (1 << 0) else False
        failed = True if flags & (1 << 2) else False
        amount = Long.read(b)
        
        date = Int.read(b)
        
        provider = String.read(b)
        
        transaction_date = Int.read(b) if flags & (1 << 1) else None
        transaction_url = String.read(b) if flags & (1 << 1) else None
        return BroadcastRevenueTransactionWithdrawal(amount=amount, date=date, provider=provider, pending=pending, failed=failed, transaction_date=transaction_date, transaction_url=transaction_url)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.pending else 0
        flags |= (1 << 2) if self.failed else 0
        flags |= (1 << 1) if self.transaction_date is not None else 0
        flags |= (1 << 1) if self.transaction_url is not None else 0
        b.write(Int(flags))
        
        b.write(Long(self.amount))
        
        b.write(Int(self.date))
        
        b.write(String(self.provider))
        
        if self.transaction_date is not None:
            b.write(Int(self.transaction_date))
        
        if self.transaction_url is not None:
            b.write(String(self.transaction_url))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from typing import Optional

from pyrogram import types, raw
from ..object import Object


class BusinessInfo(Object):
    """Business information of a user.

    Parameters:
        address (``str``, *optional*):
            Address of the business.

        location (:obj:`~pyrogram.types.Location`, *optional*):
            Location of the business on the map.

        greeting_message (:obj:`~pyrogram.types.BusinessMessage`, *optional*):
            Greeting message of the business.

        away_message (:obj:`~pyrogram.types.BusinessMessage`, *optional*):
            Away message of the business.

        working_hours (:obj:`~pyrogram.types.BusinessWorkingHours`, *optional*):
            Working hours of the business.
    """

    def __init__(
        self,
        *,
        address: str = None,
        location: "types.Location" = None,
        greeting_message: "types.BusinessMessage" = None,
        away_message: "types.BusinessMessage" = None,
        working_hours: "types.BusinessWorkingHours" = None,

    ):
        self.address = address
        self.location = location
        self.greeting_message = greeting_message
        self.away_message = away_message
        self.working_hours = working_hours

    @staticmethod
    def _parse(
        client,
        user: "raw.types.UserFull" = None,
        users: dict = None
    ) -> Optional["BusinessInfo"]:
        working_hours = getattr(user, "business_work_hours", None)
        location = getattr(user, "business_location", None)
        greeting_message = getattr(user, "business_greeting_message", None)
        away_message = getattr(user, "business_away_message", None)

        if not any((working_hours, location, greeting_message, away_message)):
            return None

        return BusinessInfo(
            address=getattr(location, "address", None),
            location=types.Location._parse(client, getattr(location, "geo_point", None)),
            greeting_message=types.BusinessMessage._parse(client, greeting_message, users),
            away_message=types.BusinessMessage._parse(client, away_message, users),
            working_hours=types.BusinessWorkingHours._parse(working_hours),
        )



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class CreateChat(TLObject):  # type: ignore
    """Telegram API function.

    Details:
        - Layer: ``177``
        - ID: ``92CEDDD4``

    Parameters:
        users (List of :obj:`InputUser <pyrogram.raw.base.InputUser>`):
            N/A

        title (``str``):
            N/A

        ttl_period (``int`` ``32-bit``, *optional*):
            N/A

    Returns:
        :obj:`messages.InvitedUsers <pyrogram.raw.base.messages.InvitedUsers>`
    """

    __slots__: List[str] = ["users", "title", "ttl_period"]

    ID = 0x92ceddd4
    QUALNAME = "functions.messages.CreateChat"

    def __init__(self, *, users: List["raw.base.InputUser"], title: str, ttl_period: Optional[int] = None) -> None:
        self.users = users  # Vector<InputUser>
        self.title = title  # string
        self.ttl_period = ttl_period  # flags.0?int

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "CreateChat":
        
        flags = Int.read(b)
        
        users = TLObject.read(b)
        
        title = String.read(b)
        
        ttl_period = Int.read(b) if flags & (1 << 0) else None
        return CreateChat(users=users, title=title, ttl_period=ttl_period)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        flags = 0
        flags |= (1 << 0) if self.ttl_period is not None else 0
        b.write(Int(flags))
        
        b.write(Vector(self.users))
        
        b.write(String(self.title))
        
        if self.ttl_period is not None:
            b.write(Int(self.ttl_period))
        
        return b.getvalue()



#  FORK PYROFORK <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyrogram.raw.core.primitives import Int, Long, Int128, Int256, Bool, Bytes, String, Double, Vector
from pyrogram.raw.core import TLObject
from pyrogram import raw
from typing import List, Optional, Any

# # # # # # # # # # # # # # # # # # # # # # # #
#               !!! WARNING !!!               #
#          This is a generated file!          #
# All changes made in this file will be lost! #
# # # # # # # # # # # # # # # # # # # # # # # #


class InputBotInlineResultGame(TLObject):  # type: ignore
    """Telegram API type.

    Constructor of :obj:`~pyrogram.raw.base.InputBotInlineResult`.

    Details:
        - Layer: ``177``
        - ID: ``4FA417F2``

    Parameters:
        id (``str``):
            N/A

        short_name (``str``):
            N/A

        send_message (:obj:`InputBotInlineMessage <pyrogram.raw.base.InputBotInlineMessage>`):
            N/A

    """

    __slots__: List[str] = ["id", "short_name", "send_message"]

    ID = 0x4fa417f2
    QUALNAME = "types.InputBotInlineResultGame"

    def __init__(self, *, id: str, short_name: str, send_message: "raw.base.InputBotInlineMessage") -> None:
        self.id = id  # string
        self.short_name = short_name  # string
        self.send_message = send_message  # InputBotInlineMessage

    @staticmethod
    def read(b: BytesIO, *args: Any) -> "InputBotInlineResultGame":
        # No flags
        
        id = String.read(b)
        
        short_name = String.read(b)
        
        send_message = TLObject.read(b)
        
        return InputBotInlineResultGame(id=id, short_name=short_name, send_message=send_message)

    def write(self, *args) -> bytes:
        b = BytesIO()
        b.write(Int(self.ID, False))

        # No flags
        
        b.write(String(self.id))
        
        b.write(String(self.short_name))
        
        b.write(self.send_message.write())
        
        return b.getvalue()
